@@ -1,5 +1,3 @@
-/* $Xorg: Xrm.c,v 1.7 2001/02/09 02:03:39 xorgcvs Exp $ */
-
 /***********************************************************
 Copyright 1987, 1988, 1990 by Digital Equipment Corporation, Maynard
 
@@ -49,7 +47,7 @@ other dealings in this Software without prior written authorization
 from The Open Group.
 
 */
-/* $XFree86: xc/lib/X11/Xrm.c,v 3.23 2003/11/17 22:20:11 dawes Exp $ */
+/* $XFree86: xc/lib/X11/Xrm.c,v 3.25 2006/01/09 14:58:40 dawes Exp $ */
 
 #include	<stdio.h>
 #include	<ctype.h>
@@ -348,6 +346,11 @@ void XrmSetDatabase(
     XrmDatabase database)
 {
     LockDisplay(display);
+    /* destroy database if set up imlicitely by XGetDefault() */
+    if (display->db && (display->flags & XlibDisplayDfltRMDB)) {
+	XrmDestroyDatabase(display->db);
+	display->flags &= ~XlibDisplayDfltRMDB;
+    }
     display->db = database;
     UnlockDisplay(display);
 }
