@@ -1,4 +1,3 @@
-/* $Xorg: sunInit.c,v 1.4 2000/08/17 19:48:29 cpqbld Exp $ */
 /*
  * sunInit.c --
  *	Initialization functions for screen/keyboard/mouse, etc.
@@ -15,7 +14,7 @@
  *
  *
  */
-/* $XFree86: xc/programs/Xserver/hw/sun/sunInit.c,v 3.14 2004/06/02 22:43:00 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/sun/sunInit.c,v 3.17 2006/02/20 17:02:02 tsi Exp $ */
 
 /************************************************************
 Copyright 1987 by Sun Microsystems, Inc. Mountain View, CA.
@@ -278,7 +277,7 @@ static PixmapFormatRec	formats[] = {
 #if SUNMAXDEPTH > 1
     ,{ 8, 8, BITMAP_SCANLINE_PAD} /* 8-bit deep */
 #if SUNMAXDEPTH > 8
-    ,{ 12, 24, BITMAP_SCANLINE_PAD } /* 12-bit deep */
+    ,{ 12, 16, BITMAP_SCANLINE_PAD } /* 12-bit deep */
     ,{ 24, 32, BITMAP_SCANLINE_PAD } /* 24-bit deep */
 #endif
 #endif
@@ -942,28 +941,3 @@ sunCfbScreenInit(pScreen, pbits, xsize, ysize, dpix, dpiy, width, bpp)
 
 #endif  /* SUNMAXDEPTH == 32 */
 #endif  /* SUNMAXDEPTH */
-
-#ifdef DPMSExtension
-/**************************************************************
- * DPMSSet(), DPMSGet(), DPMSSupported()
- *
- * stubs
- *
- ***************************************************************/
-
-void DPMSSet (level)
-    int level;
-{
-}
-
-int DPMSGet (level)
-    int* level;
-{
-    return -1;
-}
-
-Bool DPMSSupported ()
-{
-    return FALSE;
-}
-#endif
