@@ -1,4 +1,3 @@
-/* $Xorg: sunLyInit.c,v 1.3 2000/08/17 19:48:36 cpqbld Exp $ */
 /*
  * This is sunInit.c modified for LynxOS
  * Copyright 1996 by Thomas Mueller
@@ -22,7 +21,7 @@
  * PERFORMANCE OF THIS SOFTWARE.
  *
  */
-/* $XFree86: xc/programs/Xserver/hw/sunLynx/sunLyInit.c,v 3.9 2003/11/17 22:20:37 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/sunLynx/sunLyInit.c,v 3.12 2006/02/19 15:51:21 tsi Exp $ */
 
 /*
  * Copyright 1987 by the Regents of the University of California
@@ -431,6 +430,12 @@ static void getKbdType()
 #endif
 }
 
+void OsVendorPreInit(
+    void
+)
+{
+}
+
 void OsVendorInit(
     void
 )
@@ -621,28 +626,3 @@ sunCfbScreenInit(pScreen, pbits, xsize, ysize, dpix, dpiy, width, bpp)
     return cfbScreenInit(pScreen, pbits, xsize, ysize, dpix, dpiy, width);
 }
 #endif  /* SUNMAXDEPTH */
-
-#ifdef DPMSExtension
-/**************************************************************
- * DPMSSet(), DPMSGet(), DPMSSupported()
- *
- * stubs
- *
- ***************************************************************/
-
-void DPMSSet (level)
-    int level;
-{
-}
-
-int DPMSGet (level)
-    int* level;
-{
-    return -1;
-}
-
-Bool DPMSSupported ()
-{
-    return FALSE;
-}
-#endif
