@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/common/xf86Init.c,v 3.228 2005/03/07 17:32:40 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/common/xf86Init.c,v 3.240 2006/04/08 18:30:26 dawes Exp $ */
 
 /*
  * Loosely based on code bearing the following copyright:
@@ -6,7 +6,7 @@
  *   Copyright 1990,91 by Thomas Roell, Dinkelscherben, Germany.
  */
 /*
- * Copyright (c) 1992-2005 by The XFree86 Project, Inc.
+ * Copyright (c) 1992-2006 by The XFree86 Project, Inc.
  * All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining
@@ -102,7 +102,7 @@
 #include <stdlib.h>
 
 #undef HAS_UTSNAME
-#if !defined(WIN32) && !defined(__UNIXOS2__)
+#if !defined(WIN32)
 #define HAS_UTSNAME 1
 #include <sys/utsname.h>
 #endif
@@ -111,10 +111,10 @@
 #ifdef __UNIXOS2__
 #define I_NEED_OS2_H
 #endif
-#include "X.h"
-#include "Xmd.h"
-#include "Xproto.h"
-#include "Xatom.h"
+#include <X11/X.h>
+#include <X11/Xmd.h>
+#include <X11/Xproto.h>
+#include <X11/Xatom.h>
 #include "input.h"
 #include "servermd.h"
 #include "windowstr.h"
@@ -141,8 +141,8 @@
 #include "xf86Build.h"
 #include "mipointer.h"
 #ifdef XINPUT
-#include "XI.h"
-#include "XIproto.h"
+#include <X11/extensions/XI.h>
+#include <X11/extensions/XIproto.h>
 #else
 #include "inputstr.h"
 #endif
@@ -160,6 +160,15 @@
 extern int xtest_command_key;
 #endif /* XTESTEXT1 */
 
+#if !defined(LOADERTEST) || !defined(XFree86LOADER)
+#undef LOADERTEST
+#define LOADERTEST 0
+#endif
+
+#if LOADERTEST
+#include "loadertest.h"
+static Bool doLoaderTest = FALSE;
+#endif
 
 /* forward declarations */
 
@@ -175,11 +184,6 @@ static Bool noVT = FALSE;
 
 static char *cmdline = NULL;
 
-#ifdef DO_CHECK_BETA
-static int extraDays = 0;
-static char *expKey = NULL;
-#endif
-
 #ifdef __UNIXOS2__
 extern void os2ServerVideoAccess();
 #endif
@@ -307,34 +311,6 @@ PostConfigInit(void)
 
     done = TRUE;
 
-    /*
-     * Install signal handler for unexpected signals
-     */
-    xf86Info.caughtSignal=FALSE;
-    if (!xf86Info.notrapSignals) {
-       signal(SIGSEGV,xf86SigHandler);
-       signal(SIGILL,xf86SigHandler);
-#ifdef SIGEMT
-       signal(SIGEMT,xf86SigHandler);
-#endif
-       signal(SIGFPE,xf86SigHandler);
-#ifdef SIGBUS
-       signal(SIGBUS,xf86SigHandler);
-#endif
-#ifdef SIGSYS
-       signal(SIGSYS,xf86SigHandler);
-#endif
-#ifdef SIGXCPU
-       signal(SIGXCPU,xf86SigHandler);
-#endif
-#ifdef SIGXFSZ
-       signal(SIGXFSZ,xf86SigHandler);
-#endif
-#ifdef MEMDEBUG
-       signal(SIGUSR2,xf86SigMemDebug);
-#endif
-    }
-
     xf86OSPMClose = xf86OSPMOpen();
     
     if (!noVT) {
@@ -354,6 +330,7 @@ InitOutput(ScreenInfo *pScreenInfo, int argc, char **argv)
 #ifdef XFree86LOADER
   const char             **modulelist;
   pointer                *optionlist;
+  Bool			 haveScreens = FALSE;
 #endif
   const char             **driverlist;
   screenLayoutPtr	 layout;
@@ -386,10 +363,6 @@ InitOutput(ScreenInfo *pScreenInfo, int argc, char **argv)
     else
       xf86ServerName = argv[0];
 
-#ifdef DO_CHECK_BETA
-    xf86CheckBeta(extraDays, expKey);
-#endif
-
     /* Set default paths. */
     xf86FileCmdline.handle = &xf86FileCmdline;
     xf86FileDefaults.handle = &xf86FileDefaults;
@@ -444,6 +417,35 @@ InitOutput(ScreenInfo *pScreenInfo, int argc, char **argv)
 	appendauto = FALSE;
     }
 
+    /*
+     * Install signal handlers earlier to catch hardware problems.  Perhaps
+     * there should be a command line flag to control xf86Info.notrapSignals.
+     */
+    xf86Info.caughtSignal=FALSE;
+    if (!xf86Info.notrapSignals) {
+       signal(SIGSEGV,xf86SigHandler);
+       signal(SIGILL,xf86SigHandler);
+#ifdef SIGEMT
+       signal(SIGEMT,xf86SigHandler);
+#endif
+       signal(SIGFPE,xf86SigHandler);
+#ifdef SIGBUS
+       signal(SIGBUS,xf86SigHandler);
+#endif
+#ifdef SIGSYS
+       signal(SIGSYS,xf86SigHandler);
+#endif
+#ifdef SIGXCPU
+       signal(SIGXCPU,xf86SigHandler);
+#endif
+#ifdef SIGXFSZ
+       signal(SIGXFSZ,xf86SigHandler);
+#endif
+#ifdef MEMDEBUG
+       signal(SIGUSR2,xf86SigMemDebug);
+#endif
+    }
+
 #ifdef XFree86LOADER
     /* Initialise the loader */
     LoaderInit();
@@ -538,7 +540,12 @@ InitOutput(ScreenInfo *pScreenInfo, int argc, char **argv)
 	LoaderFreeDirList(list);
     }
 #endif
-	
+
+#if LOADERTEST
+    if (doLoaderTest)
+	LoaderTest();
+#endif
+
     /* Load mandatory probe and base modules. */
     if (probeModules) {
 	if (!xf86LoadModules(probeModules, NULL))
@@ -702,7 +709,9 @@ retry:
      * For autoconfiguration, where several drivers will be tried in
      * sequence, we need to make sure that those with a failing Probe() up
      * until the first successful Probe() are deleted here.  This is
-     * important for the autoconfiguration retry mechanism.
+     * important for the autoconfiguration retry mechanism.  Also, stop
+     * probing at the first successful probe.  The remaining fallback
+     * drivers will be retried on preinit failure.
      */
 
     found = 0;
@@ -712,7 +721,7 @@ retry:
 	  if (xf86DriverList[i]->Probe(xf86DriverList[i], PROBE_DEFAULT))
 	    found++;
 	  if (!found) {
-	    xf86DeleteDriver(i, TRUE);
+	    xf86DeleteDriver(i);
 	  }
 	} else {
 	  xf86MsgVerb(X_WARNING, 0,
@@ -721,6 +730,8 @@ retry:
 			xf86DriverList[i]->driverName : "noname");
 	}
 	xf86SetPciVideo(NULL,NONE);
+	if (found && autoconfig)
+	  break;
       }
     }
 
@@ -821,10 +832,19 @@ retry:
     for (i = 0; i < xf86NumScreens; i++) {
 	xf86EnableAccess(xf86Screens[i]);
 	if (xf86Screens[i]->PreInit &&
-	    xf86Screens[i]->PreInit(xf86Screens[i], 0))
+	    xf86Screens[i]->PreInit(xf86Screens[i], 0)) {
 	    xf86Screens[i]->configured = TRUE;
+#ifdef XFree86LOADER
+	    haveScreens = TRUE;
+#endif
+	}
     }
 
+#ifdef XFree86LOADER
+    if (!haveScreens)
+	LoaderCheckUnresolved(0);
+#endif
+
     for (i = 0; i < xf86NumScreens; i++)
 	if (!xf86Screens[i]->configured)
 	    xf86DeleteScreen(i--, 0);
@@ -837,7 +857,7 @@ retry:
     if (xf86NumScreens == 0 && autoconfig) {
 	for (i = 0; i < xf86NumDrivers; i++) {
 	    if (xf86DriverList[i]) {
-		xf86DeleteDriver(i, TRUE);
+		xf86DeleteDriver(i);
 		autoretry = TRUE;
 		/* Clear claimed config sections. */
 		for (j = 0; j < xf86Info.serverLayout->numScreens; j++) {
@@ -911,11 +931,10 @@ retry:
     }
 
 #ifdef XFree86LOADER
-    xf86DoDeferredUnloads();
     /* Remove (unload) drivers that are not required. */
     for (i = 0; i < xf86NumDrivers; i++)
 	if (xf86DriverList[i] && xf86DriverList[i]->refCount <= 0) {
-	    xf86DeleteDriver(i, FALSE);
+	    xf86DeleteDriver(i);
 }
 #endif
 
@@ -1063,34 +1082,6 @@ retry:
     xf86EnableIO();
   }
 
-#if 0
-  /*
-   * Install signal handler for unexpected signals
-   */
-  xf86Info.caughtSignal=FALSE;
-  if (!xf86Info.notrapSignals)
-  {
-     signal(SIGSEGV,xf86SigHandler);
-     signal(SIGILL,xf86SigHandler);
-#ifdef SIGEMT
-     signal(SIGEMT,xf86SigHandler);
-#endif
-     signal(SIGFPE,xf86SigHandler);
-#ifdef SIGBUS
-     signal(SIGBUS,xf86SigHandler);
-#endif
-#ifdef SIGSYS
-     signal(SIGSYS,xf86SigHandler);
-#endif
-#ifdef SIGXCPU
-     signal(SIGXCPU,xf86SigHandler);
-#endif
-#ifdef SIGXFSZ
-     signal(SIGXFSZ,xf86SigHandler);
-#endif
-  }
-#endif
-
   /*
    * Use the previously collected parts to setup pScreenInfo
    */
@@ -1197,7 +1188,7 @@ retry:
   }
 
 #ifdef XFree86LOADER
-    if ((serverGeneration == 1) && LoaderCheckUnresolved(LD_RESOLV_IFDONE)) {
+    if ((serverGeneration == 1) && LoaderCheckUnresolved(0)) {
 	/* For now, just a warning */
 	xf86Msg(X_WARNING, "Some symbols could not be resolved!\n");
     }
@@ -1412,7 +1403,7 @@ OsVendorInit()
 #ifdef O_NONBLOCK
   if (!beenHere) {
 #if !defined(__EMX__)
-    if (geteuid() == 0 && getuid() != geteuid())
+    if (PRIVS_ELEVATED)
 #endif
     {
       int status;
@@ -1460,9 +1451,6 @@ ddxGiveUp()
 #ifdef USE_XF86_SERVERLOCK
     xf86UnlockServer();
 #endif
-#ifdef XFreeXDGA
-    DGAShutdown();
-#endif
 
     if (!noVT)
 	xf86CloseConsole();
@@ -1470,8 +1458,9 @@ ddxGiveUp()
     xf86CloseLog();
 
     /* If an unexpected signal was caught, dump a core for debugging */
-    if (xf86Info.caughtSignal)
+    if (xf86Info.caughtSignal) {
 	abort();
+    }
 }
 
 
@@ -1503,9 +1492,12 @@ AbortDDX()
     sleep(1);
 #endif
   if (xf86Screens && !noVT) {
+#ifdef XFreeXDGA
+    DGAShutdown();
+#endif
     if (xf86Screens[0]->vtSema)
       xf86EnterServerState(SETUP);
-    for (i = 0; i < xf86NumScreens; i++)
+    for (i = 0; i < xf86NumScreens; i++) {
       if (xf86Screens[i]->vtSema) {
 	/*
 	 * If we are aborting before ScreenInit() has finished
@@ -1515,6 +1507,7 @@ AbortDDX()
 	xf86EnableAccess(xf86Screens[i]);
 	(xf86Screens[i]->LeaveVT)(i, 0);
       }
+    }
   }
   
   xf86AccessLeave();
@@ -1598,7 +1591,7 @@ ddxProcessArgument(int argc, char **argv, int i)
   }
 
   /* First the options that are only allowed for root */
-  if (getuid() == 0)
+  if (!PRIVS_ELEVATED)
   {
     if (!strcmp(argv[i], "-modulepath")) {
       if (!argv[++i])
@@ -1624,7 +1617,7 @@ ddxProcessArgument(int argc, char **argv, int i)
   {
     if (!argv[++i])
       return 0;
-    if (getuid() != 0 && !xf86PathIsSafe(argv[i])) {
+    if (PRIVS_ELEVATED && !xf86PathIsSafe(argv[i])) {
       FatalError("\nInvalid argument for -xf86config\n"
 	  "\tFor non-root users, the file specified with -xf86config must be\n"
 	  "\ta relative path and must not contain any \"..\" elements.\n"
@@ -1704,12 +1697,20 @@ ddxProcessArgument(int argc, char **argv, int i)
 #endif
     return 1;
   }
-#ifdef DO_CHECK_BETA
-  if (!strcmp(argv[i],"-extendExpiry"))
+#ifdef XFree86LOADER
+#if LOADERTEST
+  if (!strcmp(argv[i],"-loadertest"))
   {
-    extraDays = atoi(argv[i + 1]);
-    expKey = argv[i + 2];
-    return 3;
+    doLoaderTest = TRUE;
+    return 1;
+  }
+#endif
+  if (!strcmp(argv[i],"-loaderdebug"))
+  {
+    if (++i >= argc)
+      return 0;
+    LoaderSetDebug(atoi(argv[i]));
+    return 2;
   }
 #endif
   if (!strcmp(argv[i],"-verbose"))
@@ -1940,7 +1941,7 @@ ddxProcessArgument(int argc, char **argv, int i)
   }
   if (!strcmp(argv[i], "-configure"))
   {
-    if (getuid() != 0) {
+    if (PRIVS_ELEVATED) {
 	ErrorF("The '-configure' option can only be used by root.\n");
 	exit(1);
     }
@@ -1964,7 +1965,7 @@ ddxUseMsg()
   ErrorF("\n");
   ErrorF("\n");
   ErrorF("Device Dependent Usage\n");
-  if (getuid() == 0)
+  if (!PRIVS_ELEVATED)
   {
     ErrorF("-xf86config file       specify a configuration file\n");
     ErrorF("-modulepath paths      specify the module search path\n");
