@@ -1,6 +1,6 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/ati/atiproto.h,v 1.10 2004/12/31 16:07:07 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/ati/atiproto.h,v 1.12 2006/01/05 18:55:30 tsi Exp $ */
 /*
- * Copyright 1997 through 2005 by Marc Aurele La France (TSI @ UQV), tsi@xfree86.org
+ * Copyright 1997 through 2006 by Marc Aurele La France (TSI @ UQV), tsi@xfree86.org
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -24,7 +24,7 @@
 #ifndef ___ATIPROTO_H___
 #define ___ATIPROTO_H___ 1
 
-#include "Xfuncproto.h"
+#include <X11/Xfuncproto.h>
 
 /*
  * This isn't quite ready for Xfuncproto.h yet.
