@@ -21,7 +21,7 @@
  *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  */
  
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/glint/pm2_video.c,v 1.26 2003/11/10 18:22:20 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/glint/pm2_video.c,v 1.28 2005/10/14 15:16:40 tsi Exp $ */
 
 #include "xf86.h"
 #include "xf86_OSproc.h"
@@ -31,7 +31,7 @@
 #include "xf86fbman.h"
 #include "xf86i2c.h"
 #include "xf86xv.h"
-#include "Xv.h"
+#include <X11/extensions/Xv.h>
 
 #include "glint_regs.h"
 #include "glint.h"
@@ -1488,8 +1488,6 @@ StartVideoStream(PortPrivPtr pPPriv, RegionPtr pRegion)
 
 	return pPPriv->StreamOn = TRUE;
     }
-
-    return FALSE;
 }
 
 
@@ -2554,8 +2552,6 @@ xvipcHandshake(PortPrivPtr pPPriv, int op, Bool block)
 	    xvipc.block = block;
 	}
     }
-
-    return TRUE;
 }
 
 static void
