@@ -25,7 +25,7 @@ TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 **************************************************************************/
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/i740/i740_driver.c,v 1.53 2005/02/18 02:55:07 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/i740/i740_driver.c,v 1.60 2006/03/16 16:50:07 dawes Exp $ */
 
 /*
  * Authors:
@@ -75,31 +75,12 @@ SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #include "micmap.h"
 
-#define USE_FB
-
-#ifdef USE_FB
 #include "fb.h"
-#else
-/* Drivers using cfb need: */
-
-#define PSZ 8
-#include "cfb.h"
-#undef PSZ
-
-/* Drivers supporting bpp 16, 24 or 32 with cfb need one or more of: */
-
-#include "cfb16.h"
-#include "cfb24.h"
-#include "cfb32.h"
-#endif
-
-/* The driver's own header file: */
-
 
 #include "regionstr.h"
 
 #include "xf86xv.h"
-#include "Xv.h"
+#include <X11/extensions/Xv.h>
 
 #include "vbe.h"
 #include "i740_dga.h"
@@ -224,17 +205,8 @@ static const char *vgahwSymbols[] = {
 
 #ifdef XFree86LOADER
 static const char *fbSymbols[] = {
-#ifdef USE_FB
     "fbScreenInit",
     "fbPictureInit",
-#else
-    "cfbScreenInit",
-    "cfb16ScreenInit",
-    "cfb24ScreenInit",
-    "cfb32ScreenInit",
-#endif
-    "cfb8_32ScreenInit",
-    "cfb24_32ScreenInit",
     NULL
 };
 #endif
@@ -253,14 +225,12 @@ static const char *ramdacSymbols[] = {
     NULL
 };
 
-#ifdef XFree86LOADER
 static const char *vbeSymbols[] = {
     "VBEInit",
     "vbeDoEDID",
     "vbeFree",
     NULL
 };
-#endif
 
 #if USE_DDC2
 static const char *ddcSymbols[] = {
@@ -299,7 +269,7 @@ static XF86ModuleVersionInfo i740VersRec =
 XF86ModuleData i740ModuleData = {&i740VersRec, i740Setup, 0};
 
 static pointer
-i740Setup(pointer module, pointer opts, int *errmaj, int *errmin)
+i740Setup(ModuleDescPtr module, pointer opts, int *errmaj, int *errmin)
 {
     static Bool setupDone = FALSE;
 
@@ -318,12 +288,12 @@ i740Setup(pointer module, pointer opts, int *errmaj, int *errmin)
 	 * Tell the loader about symbols from other modules that this module
 	 * might refer to.
 	 */
-	LoaderRefSymLists(vgahwSymbols, fbSymbols, xaaSymbols, 
-			  ramdacSymbols, vbeSymbols,
+	LoaderModRefSymLists(module, vgahwSymbols, fbSymbols, xaaSymbols, 
+			     ramdacSymbols, vbeSymbols,
 #if USE_DDC2
-			  ddcSymbols, i2cSymbols,
+			     ddcSymbols, i2cSymbols,
 #endif
-			  NULL);
+			     NULL);
 
 	/*
 	 * The return value must be non-NULL on success even though there
@@ -479,10 +449,14 @@ static void
 I740ProbeDDC(ScrnInfoPtr pScrn, int index)
 {
     vbeInfoPtr pVbe;
-    if (xf86LoadSubModule(pScrn, "vbe")) {
+    ModuleDescPtr pMod;
+
+    if ((pMod = xf86LoadVBEModule(pScrn))) {
+	xf86LoaderModReqSymLists(pMod, vbeSymbols, NULL);
 	pVbe = VBEInit(NULL,index);
 	ConfiguredMonitor = vbeDoEDID(pVbe, NULL);
 	vbeFree(pVbe);
+	xf86UnloadSubModule(pMod);
     }
 }
 
@@ -500,11 +474,9 @@ I740PreInit(ScrnInfoPtr pScrn, int flags) {
   int i;
   MessageType from;
   int temp;
-#ifndef USE_FB
-  char *mod=0, *reqSym=0;
-#endif
   int flags24;
   rgb defaultWeight = {0, 0, 0};
+  ModuleDescPtr pMod;
 
   if (pScrn->numEntities != 1) return FALSE;
 
@@ -525,9 +497,9 @@ I740PreInit(ScrnInfoPtr pScrn, int flags) {
   }
 
   /* The vgahw module should be loaded here when needed */
-  if (!xf86LoadSubModule(pScrn, "vgahw")) return FALSE;
+  if (!(pMod = xf86LoadSubModule(pScrn, "vgahw"))) return FALSE;
 
-  xf86LoaderReqSymLists(vgahwSymbols, NULL);
+  xf86LoaderModReqSymLists(pMod, vgahwSymbols, NULL);
 
   /* Allocate a vgaHWRec */
   if (!vgaHWGetHWRec(pScrn)) return FALSE;
@@ -705,7 +677,7 @@ I740PreInit(ScrnInfoPtr pScrn, int flags) {
     }
   }
 
-  xf86DrvMsg(pScrn->scrnIndex, from, "Steve was here! VideoRAM: %d kByte %s\n",
+  xf86DrvMsg(pScrn->scrnIndex, from, "VideoRAM: %d kByte %s\n",
 	     pScrn->videoRam, (pI740->HasSGRAM)?"SGRAM":"SDRAM");
   pI740->FbMapSize = pScrn->videoRam*1024;
 
@@ -795,52 +767,26 @@ I740PreInit(ScrnInfoPtr pScrn, int flags) {
 
   xf86SetDpi(pScrn, 0, 0);
 
-#ifdef USE_FB
-  if (!xf86LoadSubModule(pScrn, "fb")) {
+  if (!(pMod = xf86LoadSubModule(pScrn, "fb"))) {
     I740FreeRec(pScrn);
     return FALSE;
   }
-  xf86LoaderReqSymbols("fbScreenInit","fbPictureInit", NULL);
-#else
-  switch (pScrn->bitsPerPixel) {
-  case 8:
-    mod = "cfb";
-    reqSym = "cfbScreenInit";
-    break;
-  case 16:
-    mod = "cfb16";
-    reqSym = "cfb16ScreenInit";
-    break;
-  case 24:
-    mod = "cfb24";
-    reqSym = "cfb24ScreenInit";
-    break;
-  case 32:
-    mod = "cfb32";
-    reqSym = "cfb32ScreenInit";
-    break;
-  }
-  if (mod && !xf86LoadSubModule(pScrn, mod)) {
-    I740FreeRec(pScrn);
-    return FALSE;
-  }
-  xf86LoaderReqSymbols(reqSym, NULL);
-#endif
+  xf86LoaderModReqSymbols(pMod, "fbScreenInit","fbPictureInit", NULL);
 
   if (!xf86ReturnOptValBool(pI740->Options, OPTION_NOACCEL, FALSE)) {
-    if (!xf86LoadSubModule(pScrn, "xaa")) {
+    if (!(pMod = xf86LoadSubModule(pScrn, "xaa"))) {
       I740FreeRec(pScrn);
       return FALSE;
     }
-    xf86LoaderReqSymLists(xaaSymbols, NULL);
+    xf86LoaderModReqSymLists(pMod, xaaSymbols, NULL);
   }
 
   if (!xf86ReturnOptValBool(pI740->Options, OPTION_SW_CURSOR, FALSE)) {
-    if (!xf86LoadSubModule(pScrn, "ramdac")) {
+    if (!(pMod = xf86LoadSubModule(pScrn, "ramdac"))) {
       I740FreeRec(pScrn);
       return FALSE;
     }
-    xf86LoaderReqSymLists(ramdacSymbols, NULL);
+    xf86LoaderModReqSymLists(pMod, ramdacSymbols, NULL);
   }
 
   /*  We wont be using the VGA access after the probe */
@@ -870,10 +816,10 @@ I740PreInit(ScrnInfoPtr pScrn, int flags) {
 #if USE_DDC2 /*DDC2*/
   { /*PL*/
 
-   if (xf86LoadSubModule(pScrn, "ddc")) {
-     xf86LoaderReqSymLists(ddcSymbols, NULL);
-     if ( xf86LoadSubModule(pScrn, "i2c") ) {
-       xf86LoaderReqSymLists(i2cSymbols,NULL);
+   if ((pMod = xf86LoadSubModule(pScrn, "ddc"))) {
+     xf86LoaderModReqSymLists(pMod, ddcSymbols, NULL);
+     if ((pMod = xf86LoadSubModule(pScrn, "i2c"))) {
+       xf86LoaderModReqSymLists(pMod, i2cSymbols,NULL);
 
        if (I740MapMem(pScrn))
 	 {
@@ -974,7 +920,7 @@ DoSave(ScrnInfoPtr pScrn, vgaRegPtr vgaReg, I740RegPtr i740Reg, Bool saveFonts)
    * in the generic VGA portion.
    */
   if (saveFonts)
-    vgaHWSave(pScrn, vgaReg, VGA_SR_MODE|VGA_SR_FONTS);
+    vgaHWSave(pScrn, vgaReg, VGA_SR_ALL);
   else
     vgaHWSave(pScrn, vgaReg, VGA_SR_MODE);
 
@@ -1086,7 +1032,7 @@ DoRestore(ScrnInfoPtr pScrn, vgaRegPtr vgaReg, I740RegPtr i740Reg,
    *		restore clock-select bits.
    */
   if (restoreFonts)
-    vgaHWRestore(pScrn, vgaReg, VGA_SR_FONTS|VGA_SR_MODE);
+    vgaHWRestore(pScrn, vgaReg, VGA_SR_ALL);
   else
     vgaHWRestore(pScrn, vgaReg, VGA_SR_MODE);
 
@@ -1597,12 +1543,9 @@ I740ScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv) {
   if (!miSetVisualTypes(pScrn->depth, miGetDefaultVisualMask(pScrn->depth),
 			pScrn->rgbBits, pScrn->defaultVisual))
     return FALSE;
-#ifdef USE_FB
 	if (!miSetPixmapDepths ()) return FALSE;
-#endif
 
   switch (pScrn->bitsPerPixel) {
-#ifdef USE_FB
   case 8:
   case 16:
   case 24:
@@ -1613,45 +1556,13 @@ I740ScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv) {
 		       pScrn->displayWidth,pScrn->bitsPerPixel))
       return FALSE;
     break;
-#else
-  case 8:
-    if (!cfbScreenInit(pScreen, pI740->FbBase, 
-		       pScrn->virtualX, pScrn->virtualY,
-		       pScrn->xDpi, pScrn->yDpi,
-		       pScrn->displayWidth))
-      return FALSE;
-    break;
-  case 16:
-    if (!cfb16ScreenInit(pScreen, pI740->FbBase, 
-			 pScrn->virtualX, pScrn->virtualY,
-			 pScrn->xDpi, pScrn->yDpi,
-			 pScrn->displayWidth))
-      return FALSE;
-    break;
-  case 24:
-    if (!cfb24ScreenInit(pScreen, pI740->FbBase, 
-			 pScrn->virtualX, pScrn->virtualY,
-			 pScrn->xDpi, pScrn->yDpi,
-			 pScrn->displayWidth))
-      return FALSE;
-    break;
-  case 32:
-    if (!cfb32ScreenInit(pScreen, pI740->FbBase, 
-			 pScrn->virtualX, pScrn->virtualY,
-			 pScrn->xDpi, pScrn->yDpi,
-			 pScrn->displayWidth))
-      return FALSE;
-    break;
-#endif
   default:
     xf86DrvMsg(scrnIndex, X_ERROR,
 	       "Internal error: invalid bpp (%d) in I740ScrnInit\n",
 	       pScrn->bitsPerPixel);
     return FALSE;
   }
-#ifdef USE_FB
   fbPictureInit(pScreen,0,0);
-#endif
 
   xf86SetBlackWhitePixels(pScreen);
 
