@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/mga/mga_dac3026.c,v 1.58 2003/11/03 05:11:17 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/mga/mga_dac3026.c,v 1.61 2004/11/26 11:48:47 tsi Exp $ */
 /*
  * Copyright 1994 by Robin Cutshaw <robin@XFree86.org>
  *
@@ -571,7 +571,9 @@ MGA3026Init(ScrnInfoPtr pScrn, DisplayModePtr mode)
 	}
 
 	/*
-	 * This will initialize all of the generic VGA registers.
+	 * This will initialize all of the generic VGA registers.  Don't bother
+	 * with VGA_FIX_SYNC_PULSES, given the relevant CRTC settings are
+	 * overridden below.  Ditto for the KGA workaround.
 	 */
 	if (!vgaHWInit(pScrn, mode))
 		return(FALSE);
@@ -984,7 +986,8 @@ MGA3026_ddc1Read(ScrnInfoPtr pScrn)
 static void
 MGA3026_I2CGetBits(I2CBusPtr b, int *clock, int *data) 
 {
-  MGAPtr pMga = MGAPTR(xf86Screens[b->scrnIndex]);
+  ScrnInfoPtr pScrn = xf86Screens[b->scrnIndex];
+  MGAPtr pMga = MGAPTR(pScrn);
   unsigned char val;
 
   /* Get the result. */
@@ -1005,7 +1008,8 @@ MGA3026_I2CGetBits(I2CBusPtr b, int *clock, int *data)
 static void
 MGA3026_I2CPutBits(I2CBusPtr b, int clock, int data)
 {
-  MGAPtr pMga = MGAPTR(xf86Screens[b->scrnIndex]);
+  ScrnInfoPtr pScrn = xf86Screens[b->scrnIndex];
+  MGAPtr pMga = MGAPTR(pScrn);
   unsigned char val,drv;
 
   /* Write the values */
@@ -1031,7 +1035,7 @@ MGA3026_i2cInit(ScrnInfoPtr pScrn)
     I2CPtr = xf86CreateI2CBusRec();
     if(!I2CPtr) return FALSE;
 
-    pMga->I2C = I2CPtr;
+    pMga->DDC_Bus1 = I2CPtr;
 
     I2CPtr->BusName    = "DDC";
     I2CPtr->scrnIndex  = pScrn->scrnIndex;
