@@ -1,9 +1,5 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/nsc/nsc_gx2_driver.c,v 1.10 2003/11/03 05:11:20 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/nsc/nsc_gx2_driver.c,v 1.16 2006/03/16 16:50:11 dawes Exp $ */
 /*
- * $Workfile: nsc_gx2_driver.c $
- * $Revision: 28 $
- * $Author: Sarma $
- *
  * File Contents: This is the main module configures the interfacing 
  *                with the X server. The individual modules will be 
  *                loaded based upon the options selected from the 
@@ -145,7 +141,6 @@
 
 #define DEBUG(x)
 #define GEODE_TRACE 0
-#define CFB 0
 
 /* Includes that are used by all drivers */
 #include "xf86.h"
@@ -168,20 +163,7 @@
 #define RC_MAX_DEPTH 24
 
 /* Frame buffer stuff */
-#if CFB
-/*
- * If using cfb, cfb.h is required.  Select the others for the bpp values
- * the driver supports.
- */
-#define PSZ 8				/* needed for cfb.h */
-#include "cfb.h"
-#undef PSZ
-#include "cfb16.h"
-#include "cfb24.h"
-#include "cfb32.h"
-#else
 #include "fb.h"
-#endif
 
 #include "shadowfb.h"
 
@@ -195,15 +177,13 @@
 #include "vbe.h"
 
 /* Check for some extensions */
-#ifdef XFreeXDGA
 #define _XF86_DGA_SERVER_
-#include "extensions/xf86dgastr.h"
-#endif /* XFreeXDGA */
+#include <X11/extensions/xf86dgastr.h>
 
 #include "globals.h"
 #include "opaque.h"
 #define DPMS_SERVER
-#include "extensions/dpms.h"
+#include <X11/extensions/dpms.h>
 
 /* Our private include file (this also includes the durango headers) */
 #include "nsc.h"
@@ -269,11 +249,7 @@ extern const char *nscVgahwSymbols[];
 extern const char *nscVbeSymbols[];
 extern const char *nscInt10Symbols[];
 
-#if CFB
-extern const char *nscCfbSymbols[];
-#else
 extern const char *nscFbSymbols[];
-#endif
 extern const char *nscXaaSymbols[];
 extern const char *nscRamdacSymbols[];
 extern const char *nscShadowSymbols[];
@@ -444,11 +420,14 @@ static void
 GX2ProbeDDC(ScrnInfoPtr pScrn, int index)
 {
    vbeInfoPtr pVbe;
+   ModuleDescPtr pMod;
 
-   if (xf86LoadSubModule(pScrn, "vbe")) {
+   if ((pMod = xf86LoadVBEModule(pScrn))) {
+      xf86LoaderModReqSymLists(pMod, nscVbeSymbols, NULL);
       pVbe = VBEInit(NULL, index);
       ConfiguredMonitor = vbeDoEDID(pVbe, NULL);
       vbeFree(pVbe);
+      xf86UnloadSubModule(pMod);
    }
 }
 
@@ -474,10 +453,6 @@ GX2PreInit(ScrnInfoPtr pScreenInfo, int flags)
    MessageType from;
    int i = 0;
    GeodePtr pGeode;
-#if CFB
-   char *mod = NULL;
-   char *reqSymbol = NULL;
-#endif /* CFB */
 #if defined(STB_X)
    GAL_ADAPTERINFO sAdapterInfo;
 #endif /* STB_X */
@@ -486,6 +461,7 @@ GX2PreInit(ScrnInfoPtr pScreenInfo, int flags)
    unsigned int SupportFlags;
    const char *s;
    char **modes;
+   ModuleDescPtr pMod;
 
 #if INT10_SUPPORT
    VESAPtr pVesa;
@@ -512,18 +488,18 @@ GX2PreInit(ScrnInfoPtr pScreenInfo, int flags)
       return TRUE;
    }
 #if INT10_SUPPORT
-   if (!xf86LoadSubModule(pScreenInfo, "int10"))
+   if (!(pMod = xf86LoadSubModule(pScreenInfo, "int10")))
       return FALSE;
-   xf86LoaderReqSymLists(nscInt10Symbols, NULL);
+   xf86LoaderModReqSymLists(pMod, nscInt10Symbols, NULL);
 #endif
    pGeode->FBVGAActive = 0;		/* KFB will Knock of VGA */
 
 #if !defined(STB_X)
    /* If the vgahw module would be needed it would be loaded here */
-   if (!xf86LoadSubModule(pScreenInfo, "vgahw")) {
+   if (!(pMod = xf86LoadSubModule(pScreenInfo, "vgahw"))) {
       return FALSE;
    }
-   xf86LoaderReqSymLists(nscVgahwSymbols, NULL);
+   xf86LoaderModReqSymLists(pMod, nscVgahwSymbols, NULL);
 #endif /* STB_X */
    GeodeDebug(("GX2PreInit(1)!\n"));
    /* Do the durango hardware detection */
@@ -978,69 +954,37 @@ GX2PreInit(ScrnInfoPtr pScreenInfo, int flags)
    xf86SetDpi(pScreenInfo, 0, 0);
    GeodeDebug(("GX2PreInit(14)!\n"));
 
-#if CFB
-   /* Load bpp-specific modules */
-   mod = NULL;
-
-   /* Load bpp-specific modules */
-   switch (pScreenInfo->bitsPerPixel) {
-   case 8:
-      mod = "cfb";
-      reqSymbol = "cfbScreenInit";
-      break;
-   case 16:
-      mod = "cfb16";
-      reqSymbol = "cfb16ScreenInit";
-      break;
-   case 24:
-      mod = "cfb24";
-      reqSymbol = "cfb24ScreenInit";
-      break;
-   case 32:
-      mod = "cfb32";
-      reqSymbol = "cfb32ScreenInit";
-      break;
-   default:
-      return FALSE;
-   }
-   if (mod && xf86LoadSubModule(pScreenInfo, mod) == NULL) {
+   if (!(pMod = xf86LoadSubModule(pScreenInfo, "fb"))) {
       GX2FreeRec(pScreenInfo);
       return FALSE;
    }
 
-   xf86LoaderReqSymbols(reqSymbol, NULL);
-#else
-   if (xf86LoadSubModule(pScreenInfo, "fb") == NULL) {
-      GX2FreeRec(pScreenInfo);
-      return FALSE;
-   }
+   xf86LoaderModReqSymLists(pMod, nscFbSymbols, NULL);
 
-   xf86LoaderReqSymLists(nscFbSymbols, NULL);
-#endif
    GeodeDebug(("GX2PreInit(15)!\n"));
    if (pGeode->NoAccel == FALSE) {
-      if (!xf86LoadSubModule(pScreenInfo, "xaa")) {
+      if (!(pMod = xf86LoadSubModule(pScreenInfo, "xaa"))) {
 	 GX2FreeRec(pScreenInfo);
 	 return FALSE;
       }
-      xf86LoaderReqSymLists(nscXaaSymbols, NULL);
+      xf86LoaderModReqSymLists(pMod, nscXaaSymbols, NULL);
    }
    GeodeDebug(("GX2PreInit(16)!\n"));
    if (pGeode->HWCursor == TRUE) {
-      if (!xf86LoadSubModule(pScreenInfo, "ramdac")) {
+      if (!(pMod = xf86LoadSubModule(pScreenInfo, "ramdac"))) {
 	 GX2FreeRec(pScreenInfo);
 	 return FALSE;
       }
-      xf86LoaderReqSymLists(nscRamdacSymbols, NULL);
+      xf86LoaderModReqSymLists(pMod, nscRamdacSymbols, NULL);
    }
    GeodeDebug(("GX2PreInit(17)!\n"));
    /* Load shadowfb if needed */
    if (pGeode->ShadowFB) {
-      if (!xf86LoadSubModule(pScreenInfo, "shadowfb")) {
+      if (!(pMod = xf86LoadSubModule(pScreenInfo, "shadowfb"))) {
 	 GX2FreeRec(pScreenInfo);
 	 return FALSE;
       }
-      xf86LoaderReqSymLists(nscShadowSymbols, NULL);
+      xf86LoaderModReqSymLists(pMod, nscShadowSymbols, NULL);
    }
    GeodeDebug(("GX2PreInit(18)!\n"));
    if (xf86RegisterResources(pGeode->pEnt->index, NULL, ResExclusive)) {
@@ -1933,24 +1877,6 @@ GX2ScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
    }
 
    switch (pScreenInfo->bitsPerPixel) {
-#if CFB
-   case 8:
-      Inited = cfbScreenInit(pScreen, FBStart, width, height,
-			     pScreenInfo->xDpi, pScreenInfo->yDpi,
-			     displayWidth);
-      break;
-   case 16:
-      Inited = cfb16ScreenInit(pScreen, FBStart, width, height,
-			       pScreenInfo->xDpi, pScreenInfo->yDpi,
-			       displayWidth);
-      break;
-   case 24:
-   case 32:
-      Inited = cfb32ScreenInit(pScreen, FBStart, width, height,
-			       pScreenInfo->xDpi, pScreenInfo->yDpi,
-			       displayWidth);
-      break;
-#else
    case 8:
    case 16:
    case 24:
@@ -1959,7 +1885,7 @@ GX2ScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 			    pScreenInfo->xDpi, pScreenInfo->yDpi,
 			    displayWidth, pScreenInfo->bitsPerPixel);
       break;
-#endif
+
    default:
       xf86DrvMsg(scrnIndex, X_ERROR,
 		 "Internal error: invalid bpp (%d) in ScreenInit\n",
@@ -1991,11 +1917,9 @@ GX2ScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 	 }
       }
    }
-#if CFB
-#else
+
    /* must be after RGB ordering fixed */
    fbPictureInit(pScreen, 0, 0);
-#endif
 
    GeodeDebug(("GX2ScreenInit(6)!\n"));
    if (!pGeode->NoAccel) {
