@@ -1,4 +1,3 @@
-/* $XConsortium: nv_driver.c /main/3 1996/10/28 05:13:37 kaleb $ */
 /*
  * Copyright 1996-1997  David J. McKay
  *
@@ -24,7 +23,7 @@
 /* Hacked together from mga driver and 3.3.4 NVIDIA driver by Jarno Paananen
    <jpaana@s2.org> */
 
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/nv/nv_driver.c,v 1.120 2003/12/11 00:12:34 mvojkovi Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/nv/nv_driver.c,v 1.143 2006/03/16 16:50:11 dawes Exp $ */
 
 #include "nv_include.h"
 
@@ -82,28 +81,38 @@ DriverRec NV = {
         0
 };
 
+/* Known cards as of 2006/01/12  */
+
 static SymTabRec NVKnownChipsets[] =
 {
   { 0x12D20018, "RIVA 128" },
+
   { 0x10DE0020, "RIVA TNT" },
+
   { 0x10DE0028, "RIVA TNT2" },
   { 0x10DE002A, "Unknown TNT2" },
   { 0x10DE002C, "Vanta" },
   { 0x10DE0029, "RIVA TNT2 Ultra" },
   { 0x10DE002D, "RIVA TNT2 Model 64" },
+
   { 0x10DE00A0, "Aladdin TNT2" },
+
   { 0x10DE0100, "GeForce 256" },
   { 0x10DE0101, "GeForce DDR" },
   { 0x10DE0103, "Quadro" },
+
   { 0x10DE0110, "GeForce2 MX/MX 400" },
   { 0x10DE0111, "GeForce2 MX 100/200" },
   { 0x10DE0112, "GeForce2 Go" },
   { 0x10DE0113, "Quadro2 MXR/EX/Go" },
+
   { 0x10DE01A0, "GeForce2 Integrated GPU" },
+
   { 0x10DE0150, "GeForce2 GTS" },
   { 0x10DE0151, "GeForce2 Ti" },
   { 0x10DE0152, "GeForce2 Ultra" },
   { 0x10DE0153, "Quadro2 Pro" },
+
   { 0x10DE0170, "GeForce4 MX 460" },
   { 0x10DE0171, "GeForce4 MX 440" },
   { 0x10DE0172, "GeForce4 MX 420" },
@@ -112,38 +121,45 @@ static SymTabRec NVKnownChipsets[] =
   { 0x10DE0175, "GeForce4 420 Go" },
   { 0x10DE0176, "GeForce4 420 Go 32M" },
   { 0x10DE0177, "GeForce4 460 Go" },
+  { 0x10DE0178, "Quadro4 550 XGL" },
 #if defined(__powerpc__)
   { 0x10DE0179, "GeForce4 MX (Mac)" },
 #else
   { 0x10DE0179, "GeForce4 440 Go 64M" },
 #endif
-  { 0x10DE017D, "GeForce4 410 Go 16M" },
+  { 0x10DE017A, "Quadro NVS" },
   { 0x10DE017C, "Quadro4 500 GoGL" },
-  { 0x10DE0178, "Quadro4 550 XGL" },
-  { 0x10DE017A, "Quadro4 NVS" },
+  { 0x10DE017D, "GeForce4 410 Go 16M" },
+
   { 0x10DE0181, "GeForce4 MX 440 with AGP8X" },
   { 0x10DE0182, "GeForce4 MX 440SE with AGP8X" },
   { 0x10DE0183, "GeForce4 MX 420 with AGP8X" },
+  { 0x10DE0185, "GeForce4 MX 4000" },
   { 0x10DE0186, "GeForce4 448 Go" },
   { 0x10DE0187, "GeForce4 488 Go" },
   { 0x10DE0188, "Quadro4 580 XGL" },
 #if defined(__powerpc__)
   { 0x10DE0189, "GeForce4 MX with AGP8X (Mac)" },
 #endif
-  { 0x10DE018A, "Quadro4 280 NVS" },
+  { 0x10DE018A, "Quadro4 NVS 280 SD" },
   { 0x10DE018B, "Quadro4 380 XGL" },
+  { 0x10DE018C, "Quadro NVS 50 PCI" },
+  { 0x10DE018D, "GeForce4 448 Go" },
+
   { 0x10DE01F0, "GeForce4 MX Integrated GPU" },
+
   { 0x10DE0200, "GeForce3" },
   { 0x10DE0201, "GeForce3 Ti 200" },
   { 0x10DE0202, "GeForce3 Ti 500" },
   { 0x10DE0203, "Quadro DCC" },
+
   { 0x10DE0250, "GeForce4 Ti 4600" },
   { 0x10DE0251, "GeForce4 Ti 4400" },
-  { 0x10DE0252, "0x0252" },
   { 0x10DE0253, "GeForce4 Ti 4200" },
   { 0x10DE0258, "Quadro4 900 XGL" },
   { 0x10DE0259, "Quadro4 750 XGL" },
   { 0x10DE025B, "Quadro4 700 XGL" },
+
   { 0x10DE0280, "GeForce4 Ti 4800" },
   { 0x10DE0281, "GeForce4 Ti 4200 with AGP8X" },
   { 0x10DE0282, "GeForce4 Ti 4800 SE" },
@@ -151,54 +167,139 @@ static SymTabRec NVKnownChipsets[] =
   { 0x10DE028C, "Quadro4 700 GoGL" },
   { 0x10DE0288, "Quadro4 980 XGL" },
   { 0x10DE0289, "Quadro4 780 XGL" },
+
   { 0x10DE0301, "GeForce FX 5800 Ultra" },
   { 0x10DE0302, "GeForce FX 5800" },
   { 0x10DE0308, "Quadro FX 2000" },
   { 0x10DE0309, "Quadro FX 1000" },
+
   { 0x10DE0311, "GeForce FX 5600 Ultra" },
   { 0x10DE0312, "GeForce FX 5600" },
-  { 0x10DE0313, "0x0313"},
-  { 0x10DE0314, "GeForce FX 5600SE" },
-  { 0x10DE0316, "0x0316" },
-  { 0x10DE0317, "0x0317" },
+  { 0x10DE0314, "GeForce FX 5600XT" },
   { 0x10DE031A, "GeForce FX Go5600" },
   { 0x10DE031B, "GeForce FX Go5650" },
   { 0x10DE031C, "Quadro FX Go700" },
-  { 0x10DE031D, "0x031D" },
-  { 0x10DE031E, "0x031E" },
-  { 0x10DE031F, "0x031F" },
+
+  { 0x10DE0320, "GeForce FX 5200" },
   { 0x10DE0321, "GeForce FX 5200 Ultra" },
   { 0x10DE0322, "GeForce FX 5200" },
-  { 0x10DE0323, "GeForce FX 5200SE" },
+  { 0x10DE0323, "GeForce FX 5200LE" },
   { 0x10DE0324, "GeForce FX Go5200" },
   { 0x10DE0325, "GeForce FX Go5250" },
+  { 0x10DE0326, "GeForce FX 5500" },
+  { 0x10DE0327, "GeForce FX 5100" },
   { 0x10DE0328, "GeForce FX Go5200 32M/64M" },
 #if defined(__powerpc__)
   { 0x10DE0329, "GeForce FX 5200 (Mac)" },
-#else
-  { 0x10DE0329, "0x0329" },
 #endif
-  { 0x10DE032A, "Quadro NVS 280 PCI" },
-  { 0x10DE032B, "Quadro FX 500" },
-  { 0x10DE032C, "GeForce FX Go5300" },
+  { 0x10DE032A, "Quadro NVS 55/280 PCI" },
+  { 0x10DE032B, "Quadro FX 500/600 PCI" },
+  { 0x10DE032C, "GeForce FX Go53xx Series" },
   { 0x10DE032D, "GeForce FX Go5100" },
-  { 0x10DE032F, "0x032F" },
+
   { 0x10DE0330, "GeForce FX 5900 Ultra" },
   { 0x10DE0331, "GeForce FX 5900" },
   { 0x10DE0332, "GeForce FX 5900XT" },
   { 0x10DE0333, "GeForce FX 5950 Ultra" },
-  { 0x10DE0334, "0x0334" },
+  { 0x10DE0334, "GeForce FX 5900ZT" },
   { 0x10DE0338, "Quadro FX 3000" },
+  { 0x10DE033F, "Quadro FX 700" },
+
   { 0x10DE0341, "GeForce FX 5700 Ultra" },
   { 0x10DE0342, "GeForce FX 5700" },
-  { 0x10DE0343, "0x0343" },
-  { 0x10DE0347, "0x0347" },
-  { 0x10DE0348, "0x0348" },
-  { 0x10DE0349, "0x0349" },
-  { 0x10DE034B, "0x034B" },
-  { 0x10DE034C, "0x034C" },
+  { 0x10DE0343, "GeForce FX 5700LE" },
+  { 0x10DE0344, "GeForce FX 5700VE" },
+  { 0x10DE0347, "GeForce FX Go5700" },
+  { 0x10DE0348, "GeForce FX Go5700" },
+  { 0x10DE034C, "Quadro FX Go1000" },
   { 0x10DE034E, "Quadro FX 1100" },
-  { 0x10DE034F, "0x034F" },
+
+  { 0x10DE0040, "GeForce 6800 Ultra" },
+  { 0x10DE0041, "GeForce 6800" },
+  { 0x10DE0042, "GeForce 6800 LE" },
+  { 0x10DE0043, "GeForce 6800 XE" },
+  { 0x10DE0045, "GeForce 6800 GT" },
+  { 0x10DE0046, "GeForce 6800 GT" },
+  { 0x10DE0047, "GeForce 6800 GS" },
+  { 0x10DE0048, "GeForce 6800 XT" },
+  { 0x10DE004E, "Quadro FX 4000" },
+
+  { 0x10DE00C0, "GeForce 6800 GS" },
+  { 0x10DE00C1, "GeForce 6800" },
+  { 0x10DE00C2, "GeForce 6800 LE" },
+  { 0x10DE00C3, "GeForce 6800 XT" },
+  { 0x10DE00C8, "GeForce Go 6800" },
+  { 0x10DE00C9, "GeForce Go 6800 Ultra" },
+  { 0x10DE00CC, "Quadro FX Go1400" },
+  { 0x10DE00CD, "Quadro FX 3450/4000 SDI" },
+  { 0x10DE00CE, "Quadro FX 1400" },
+
+  { 0x10DE0140, "GeForce 6600 GT" },
+  { 0x10DE0141, "GeForce 6600" },
+  { 0x10DE0142, "GeForce 6600 LE" },
+  { 0x10DE0143, "GeForce 6600 VE" },
+  { 0x10DE0144, "GeForce Go 6600" },
+  { 0x10DE0145, "GeForce 6610 XL" },
+  { 0x10DE0146, "GeForce Go 6600 TE/6200 TE" },
+  { 0x10DE0147, "GeForce 6700 XL" },
+  { 0x10DE0148, "GeForce Go 6600" },
+  { 0x10DE0149, "GeForce Go 6600 GT" },
+  { 0x10DE014E, "Quadro FX 540" },
+  { 0x10DE014F, "GeForce 6200" },
+
+  { 0x10DE0160, "GeForce 6500" },
+  { 0x10DE0161, "GeForce 6200 TurboCache(TM)" },
+  { 0x10DE0162, "GeForce 6200SE TurboCache(TM)" },
+  { 0x10DE0163, "GeForce 6200 LE" },
+  { 0x10DE0164, "GeForce Go 6200" },
+  { 0x10DE0165, "Quadro NVS 285" },
+  { 0x10DE0166, "GeForce Go 6400" },
+  { 0x10DE0167, "GeForce Go 6200" },
+  { 0x10DE0168, "GeForce Go 6400" },
+  { 0x10DE0169, "GeForce 6250" },
+
+  { 0x10DE0211, "GeForce 6800" },
+  { 0x10DE0212, "GeForce 6800 LE" },
+  { 0x10DE0215, "GeForce 6800 GT" },
+  { 0x10DE0218, "GeForce 6800 XT" },
+
+  { 0x10DE0221, "GeForce 6200" },
+
+  { 0x10DE0090, "GeForce 7800 GTX" },
+  { 0x10DE0091, "GeForce 7800 GTX" },
+  { 0x10DE0092, "GeForce 7800 GT" },
+  { 0x10DE0093, "GeForce 7800 GS" },
+  { 0x10DE0095, "GeForce 7800 SLI" },
+  { 0x10DE0098, "GeForce Go 7800" },
+  { 0x10DE0099, "GeForce Go 7800 GTX" },
+  { 0x10DE009D, "Quadro FX 4500" },
+
+  { 0x10DE01D1, "GeForce 7300 LE" },
+  { 0x10DE01D6, "GeForce Go 7200" },
+  { 0x10DE01D7, "GeForce Go 7300" },
+  { 0x10DE01D8, "GeForce Go 7400" },
+  { 0x10DE01DA, "Quadro NVS 110M" },
+  { 0x10DE01DB, "Quadro NVS 120M" },
+  { 0x10DE01DC, "Quadro FX 350M" },
+  { 0x10DE01DE, "Quadro FX 350" },
+  { 0x10DE01DF, "GeForce 7300 GS" },
+
+  { 0x10DE0398, "GeForce Go 7600" },
+  { 0x10DE0399, "GeForce Go 7600 GT"},
+  { 0x10DE039A, "Quadro NVS 300M" },
+  { 0x10DE039C, "Quadro FX 550M" },
+
+  { 0x10DE0298, "GeForce Go 7900 GS" },
+  { 0x10DE0299, "GeForce Go 7900 GTX" },
+  { 0x10DE029A, "Quadro FX 2500M" },
+  { 0x10DE029B, "Quadro FX 1500M" },
+  
+  { 0x10DE0240, "GeForce 6150" },
+  { 0x10DE0241, "GeForce 6150 LE" },
+  { 0x10DE0242, "GeForce 6100" },
+  { 0x10DE0244, "GeForce Go 6150" },
+  { 0x10DE0247, "GeForce Go 6100" },
+
   {-1, NULL}
 };
 
@@ -207,8 +308,8 @@ static SymTabRec NVKnownChipsets[] =
  * List of symbols from other modules that this module references.  This
  * list is used to tell the loader that it is OK for symbols here to be
  * unresolved providing that it hasn't been told that they haven't been
- * told that they are essential via a call to xf86LoaderReqSymbols() or
- * xf86LoaderReqSymLists().  The purpose is this is to avoid warnings about
+ * told that they are essential via a call to xf86LoaderModReqSymbols() or
+ * xf86LoaderModReqSymLists().  The purpose is this is to avoid warnings about
  * unresolved symbols that are not required.
  */
 
@@ -257,14 +358,12 @@ static const char *ddcSymbols[] = {
     NULL
 };
 
-#ifdef XFree86LOADER
 static const char *vbeSymbols[] = {
     "VBEInit",
     "vbeFree",
     "vbeDoEDID",
     NULL
 };
-#endif
 
 static const char *i2cSymbols[] = {
     "xf86CreateI2CBusRec",
@@ -346,7 +445,9 @@ typedef enum {
     OPTION_VIDEO_KEY,
     OPTION_FLAT_PANEL,
     OPTION_FP_DITHER,
-    OPTION_CRTC_NUMBER
+    OPTION_CRTC_NUMBER,
+    OPTION_FP_SCALE,
+    OPTION_FP_TWEAK
 } NVOpts;
 
 
@@ -361,6 +462,8 @@ static const OptionInfoRec NVOptions[] = {
     { OPTION_FLAT_PANEL,	"FlatPanel",	OPTV_BOOLEAN,	{0}, FALSE },
     { OPTION_FP_DITHER, 	"FPDither",	OPTV_BOOLEAN,	{0}, FALSE },
     { OPTION_CRTC_NUMBER,	"CrtcNumber",	OPTV_INTEGER,	{0}, FALSE },
+    { OPTION_FP_SCALE,          "FPScale",      OPTV_BOOLEAN,   {0}, FALSE },
+    { OPTION_FP_TWEAK,          "FPTweak",      OPTV_INTEGER,   {0}, FALSE },
     { -1,                       NULL,           OPTV_NONE,      {0}, FALSE }
 };
 
@@ -401,7 +504,7 @@ NVFreeRec(ScrnInfoPtr pScrn)
 #ifdef XFree86LOADER
 
 static pointer
-nvSetup(pointer module, pointer opts, int *errmaj, int *errmin)
+nvSetup(ModuleDescPtr module, pointer opts, int *errmaj, int *errmin)
 {
     static Bool setupDone = FALSE;
 
@@ -420,10 +523,10 @@ nvSetup(pointer module, pointer opts, int *errmaj, int *errmin)
          * Tell the loader about symbols from other modules that this module
          * might refer to.
          */
-        LoaderRefSymLists(vgahwSymbols, xaaSymbols, fbSymbols,
-                          ramdacSymbols, shadowSymbols, rivaSymbols,
-                          i2cSymbols, ddcSymbols, vbeSymbols,
-                          fbdevHWSymbols, int10Symbols, NULL);
+        LoaderModRefSymLists(module, vgahwSymbols, xaaSymbols, fbSymbols,
+                             ramdacSymbols, shadowSymbols, rivaSymbols,
+                             i2cSymbols, ddcSymbols, vbeSymbols,
+                             fbdevHWSymbols, int10Symbols, NULL);
 
         /*
          * The return value must be non-NULL on success even though there
@@ -490,6 +593,36 @@ NVGetScrnInfoRec(PciChipsets *chips, int chip)
 
 #define MAX_CHIPS MAXSCREENS
 
+
+static CARD32 
+NVGetPCIXpressChip (pciVideoPtr pVideo)
+{
+    volatile CARD32 *regs;
+    CARD32 pciid, pcicmd;
+    PCITAG Tag = ((pciConfigPtr)(pVideo->thisCard))->tag;
+
+    pcicmd = pciReadLong(Tag, PCI_CMD_STAT_REG);
+    pciWriteLong(Tag, PCI_CMD_STAT_REG, pcicmd | PCI_CMD_MEM_ENABLE);
+    
+    regs = xf86MapPciMem(-1, VIDMEM_MMIO, Tag, pVideo->memBase[0], 0x2000);
+
+    pciid = regs[0x1800/4];
+
+    xf86UnMapVidMem(-1, (pointer)regs, 0x2000);
+
+    pciWriteLong(Tag, PCI_CMD_STAT_REG, pcicmd);
+
+    if((pciid & 0x0000ffff) == 0x000010DE) 
+       pciid = 0x10DE0000 | (pciid >> 16);
+    else 
+    if((pciid & 0xffff0000) == 0xDE100000) /* wrong endian */
+       pciid = 0x10DE0000 | ((pciid << 8) & 0x0000ff00) |
+                            ((pciid >> 8) & 0x000000ff);
+
+    return pciid;
+}
+
+
 /* Mandatory */
 static Bool
 NVProbe(DriverPtr drv, int flags)
@@ -503,6 +636,7 @@ NVProbe(DriverPtr drv, int flags)
     int numDevSections;
     int numUsed;
     Bool foundScreen = FALSE;
+    ModuleDescPtr pMod;
 
 
     if ((numDevSections = xf86MatchDevice(NV_DRIVER_NAME, &devSections)) <= 0) 
@@ -519,7 +653,11 @@ NVProbe(DriverPtr drv, int flags)
            ((*ppPci)->vendor == PCI_VENDOR_NVIDIA)) 
         {
             SymTabRec *nvchips = NVKnownChipsets;
-            int token = ((*ppPci)->vendor << 16) | (*ppPci)->chipType;
+            int pciid = ((*ppPci)->vendor << 16) | (*ppPci)->chipType;
+            int token = pciid;
+
+            if((token & 0xfff0) == 0x00F0) 
+                token = NVGetPCIXpressChip(*ppPci);
 
             while(nvchips->name) {
                if(token == nvchips->token)
@@ -528,10 +666,10 @@ NVProbe(DriverPtr drv, int flags)
             }
 
             if(nvchips->name) { /* found one */
-               NVChipsets[numUsed].token = nvchips->token;
+               NVChipsets[numUsed].token = pciid;
                NVChipsets[numUsed].name = nvchips->name;
-               NVPciChipsets[numUsed].numChipset = nvchips->token;
-               NVPciChipsets[numUsed].PCIid = nvchips->token;
+               NVPciChipsets[numUsed].numChipset = pciid; 
+               NVPciChipsets[numUsed].PCIid = pciid;
                NVPciChipsets[numUsed].resList = RES_SHARED_VGA;
                numUsed++;
             } else if ((*ppPci)->vendor == PCI_VENDOR_NVIDIA) {
@@ -547,10 +685,22 @@ NVProbe(DriverPtr drv, int flags)
                case 0x0320:
                case 0x0330:
                case 0x0340:
-                   NVChipsets[numUsed].token = token;
+               case 0x0040:
+               case 0x00C0:
+               case 0x0120:
+               case 0x0140:
+               case 0x0160:
+               case 0x01D0:
+               case 0x0090:
+               case 0x0210:
+               case 0x0220:
+               case 0x0240:
+               case 0x0290:
+               case 0x0390:
+                   NVChipsets[numUsed].token = pciid;
                    NVChipsets[numUsed].name = "Unknown NVIDIA chip";
-                   NVPciChipsets[numUsed].numChipset = token;
-                   NVPciChipsets[numUsed].PCIid = token;
+                   NVPciChipsets[numUsed].numChipset = pciid;
+                   NVPciChipsets[numUsed].PCIid = pciid;
                    NVPciChipsets[numUsed].resList = RES_SHARED_VGA;
                    numUsed++;
                    break;
@@ -582,10 +732,10 @@ NVProbe(DriverPtr drv, int flags)
 
         pPci = xf86GetPciInfoForEntity(usedChips[i]);
         if(pPci->vendor == PCI_VENDOR_NVIDIA_SGS) {
-            if (!xf86LoadDrvSubModule(drv, "riva128")) {
+            if (!(pMod = xf86LoadDrvSubModule(drv, "riva128"))) {
                   continue;
             }
-            xf86LoaderReqSymLists(rivaSymbols, NULL);
+            xf86LoaderModReqSymLists(pMod, rivaSymbols, NULL);
             if(RivaGetScrnInfoRec(NVPciChipsets, usedChips[i]))
                 foundScreen = TRUE;
         } else {
@@ -764,6 +914,17 @@ NVFreeScreen(int scrnIndex, int flags)
 static ModeStatus
 NVValidMode(int scrnIndex, DisplayModePtr mode, Bool verbose, int flags)
 {
+    NVPtr pNv = NVPTR(xf86Screens[scrnIndex]);
+
+    if(pNv->fpWidth && pNv->fpHeight) {
+      if((pNv->fpWidth < mode->HDisplay) || (pNv->fpHeight < mode->VDisplay)) {
+         xf86DrvMsg(scrnIndex, X_INFO, "Mode \"%s\" is larger than "
+                    "BIOS programmed panel size of %d x %d.  Removing.\n",
+                     mode->name, pNv->fpWidth, pNv->fpHeight);
+         return (MODE_PANEL);
+      }
+    }
+
     return (MODE_OK);
 }
 
@@ -771,11 +932,14 @@ static void
 nvProbeDDC(ScrnInfoPtr pScrn, int index)
 {
     vbeInfoPtr pVbe;
+    ModuleDescPtr pMod;
 
-    if (xf86LoadSubModule(pScrn, "vbe")) {
+    if ((pMod = xf86LoadVBEModule(pScrn))) {
+	xf86LoaderModReqSymLists(pMod, vbeSymbols, NULL);
         pVbe = VBEInit(NULL,index);
         ConfiguredMonitor = vbeDoEDID(pVbe, NULL);
 	vbeFree(pVbe);
+	xf86UnloadSubModule(pMod);
     }
 }
 
@@ -783,13 +947,14 @@ nvProbeDDC(ScrnInfoPtr pScrn, int index)
 Bool NVI2CInit(ScrnInfoPtr pScrn)
 {
     char *mod = "i2c";
+    ModuleDescPtr pMod;
 
-    if (xf86LoadSubModule(pScrn, mod)) {
-        xf86LoaderReqSymLists(i2cSymbols,NULL);
+    if ((pMod = xf86LoadSubModule(pScrn, mod))) {
+        xf86LoaderModReqSymLists(pMod, i2cSymbols,NULL);
 
         mod = "ddc";
-        if(xf86LoadSubModule(pScrn, mod)) {
-            xf86LoaderReqSymLists(ddcSymbols, NULL);
+        if ((pMod = xf86LoadSubModule(pScrn, mod))) {
+            xf86LoaderModReqSymLists(pMod, ddcSymbols, NULL);
             return NVDACi2cInit(pScrn);
         } 
     }
@@ -809,6 +974,7 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
     int i, max_width, max_height;
     ClockRangePtr clockRanges;
     const char *s;
+    ModuleDescPtr pMod;
 
     if (flags & PROBE_DETECT) {
         nvProbeDDC( pScrn, xf86GetEntityInfo(pScrn->entityList[0])->index );
@@ -851,8 +1017,8 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
     pNv->Primary = xf86IsPrimaryPci(pNv->PciInfo);
 
     /* Initialize the card through int10 interface if needed */
-    if (xf86LoadSubModule(pScrn, "int10")) {
- 	xf86LoaderReqSymLists(int10Symbols, NULL);
+    if ((pMod = xf86LoadSubModule(pScrn, "int10"))) {
+ 	xf86LoaderModReqSymLists(pMod, int10Symbols, NULL);
 #if !defined(__alpha__) && !defined(__powerpc__)
         xf86DrvMsg(pScrn->scrnIndex, X_INFO, "Initializing int10\n");
         pNv->pInt = xf86InitInt10(pNv->pEnt->index);
@@ -883,11 +1049,16 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
     } else {
 	from = X_PROBED;
 	pNv->Chipset = (pNv->PciInfo->vendor << 16) | pNv->PciInfo->chipType;
+
+        if((pNv->Chipset & 0xfff0) == 0x00F0)
+            pNv->Chipset = NVGetPCIXpressChip(pNv->PciInfo);
+
 	pScrn->chipset = (char *)xf86TokenToString(NVKnownChipsets, 
                                                    pNv->Chipset);
         if(!pScrn->chipset)
           pScrn->chipset = "Unknown NVIDIA chipset";
     }
+
     if (pNv->pEnt->device->chipRev >= 0) {
 	pNv->ChipRev = pNv->pEnt->device->chipRev;
 	xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "ChipRev override: %d\n",
@@ -975,12 +1146,12 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
     }
 
     /* The vgahw module should be loaded here when needed */
-    if (!xf86LoadSubModule(pScrn, "vgahw")) {
+    if (!(pMod = xf86LoadSubModule(pScrn, "vgahw"))) {
 	xf86FreeInt10(pNv->pInt);
 	return FALSE;
     }
     
-    xf86LoaderReqSymLists(vgahwSymbols, NULL);
+    xf86LoaderModReqSymLists(pMod, vgahwSymbols, NULL);
 
     /*
      * Allocate a vgaHWRec
@@ -1022,6 +1193,12 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
     }
     xf86DrvMsg(pScrn->scrnIndex, from, "Using %s cursor\n",
 		pNv->HWCursor ? "HW" : "SW");
+
+    pNv->FpScale = TRUE;
+    if (xf86GetOptValBool(pNv->Options, OPTION_FP_SCALE, &pNv->FpScale)) {
+        xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Flat panel scaling %s\n",
+                   pNv->FpScale ? "on" : "off");
+    }
     if (xf86ReturnOptValBool(pNv->Options, OPTION_NOACCEL, FALSE)) {
 	pNv->NoAccel = TRUE;
 	xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Acceleration disabled\n");
@@ -1039,12 +1216,12 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
     }
     if (pNv->FBDev) {
 	/* check for linux framebuffer device */
-	if (!xf86LoadSubModule(pScrn, "fbdevhw")) {
+	if (!(pMod = xf86LoadSubModule(pScrn, "fbdevhw"))) {
 	    xf86FreeInt10(pNv->pInt);
 	    return FALSE;
 	}
 	
-	xf86LoaderReqSymLists(fbdevHWSymbols, NULL);
+	xf86LoaderModReqSymLists(pMod, fbdevHWSymbols, NULL);
 	if (!fbdevHWInit(pScrn, pNv->PciInfo, NULL)) {
 	    xf86FreeInt10(pNv->pInt);
 	    return FALSE;
@@ -1111,6 +1288,14 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
         pNv->CRTCnumber = -1; /* autodetect later */
     }
 
+
+    if (xf86GetOptValInteger(pNv->Options, OPTION_FP_TWEAK, 
+                             &pNv->PanelTweak))
+    {
+        pNv->usePanelTweak = TRUE;
+    } else {
+        pNv->usePanelTweak = FALSE;
+    }
     
     if (pNv->pEnt->device->MemBase != 0) {
 	/* Require that the config file value matches one of the PCI values. */
@@ -1174,12 +1359,49 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
 	return FALSE;
     }
 
-    pNv->alphaCursor = ((pNv->Chipset & 0x0ff0) >= 0x0110);
+    switch (pNv->Chipset & 0x0ff0) {
+    case 0x0100:   /* GeForce 256 */
+    case 0x0110:   /* GeForce2 MX */
+    case 0x0150:   /* GeForce2 */
+    case 0x0170:   /* GeForce4 MX */
+    case 0x0180:   /* GeForce4 MX (8x AGP) */
+    case 0x01A0:   /* nForce */
+    case 0x01F0:   /* nForce2 */
+         pNv->Architecture =  NV_ARCH_10;
+         break;
+    case 0x0200:   /* GeForce3 */
+    case 0x0250:   /* GeForce4 Ti */
+    case 0x0280:   /* GeForce4 Ti (8x AGP) */
+         pNv->Architecture =  NV_ARCH_20;
+         break;
+    case 0x0300:   /* GeForce FX 5800 */
+    case 0x0310:   /* GeForce FX 5600 */
+    case 0x0320:   /* GeForce FX 5200 */
+    case 0x0330:   /* GeForce FX 5900 */
+    case 0x0340:   /* GeForce FX 5700 */
+         pNv->Architecture =  NV_ARCH_30;
+         break;
+    case 0x0040:   /* GeForce 6800 */
+    case 0x00C0:   /* GeForce 6800 */
+    case 0x0120:   /* GeForce 6800 */
+    case 0x0140:   /* GeForce 6600 */
+    case 0x0160:   /* GeForce 6200 */
+    case 0x01D0:   /* GeForce 7200, 7300, 7400 */
+    case 0x0090:   /* GeForce 7800 */
+    case 0x0210:   /* GeForce 6800 */
+    case 0x0220:   /* GeForce 6200 */
+    case 0x0290:   /* GeForce 7900 */
+    case 0x0390:   /* GeForce 7600 */
+    case 0x0240:   /* GeForce 6100 */
+         pNv->Architecture =  NV_ARCH_40;
+         break;
+    default:
+         pNv->Architecture =  NV_ARCH_04;
+         break;
+    }
 
-    pNv->Architecture = (pNv->Chipset & 0x0f00) >> 4;
-
-    if(pNv->Architecture < NV_ARCH_10)
-        pNv->Architecture = NV_ARCH_04;
+    pNv->alphaCursor = (pNv->Architecture >= NV_ARCH_10) &&
+                       ((pNv->Chipset & 0x0ff0) != 0x0100);
 
     NVCommonSetup(pScrn);
 
@@ -1207,9 +1429,13 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
 	}
     }
 
-    pNv->FbUsableSize = pNv->FbMapSize - (128 * 1024);
+    if(pNv->Architecture >= NV_ARCH_40)
+       pNv->FbUsableSize = pNv->FbMapSize - (560 * 1024);
+    else
+       pNv->FbUsableSize = pNv->FbMapSize - (128 * 1024);
     pNv->ScratchBufferSize = (pNv->Architecture < NV_ARCH_10) ? 8192 : 16384;
     pNv->ScratchBufferStart = pNv->FbUsableSize - pNv->ScratchBufferSize;
+    pNv->CursorStart = pNv->FbUsableSize + (32 * 1024);
 
     /*
      * Setup the ClockRanges, which describe what clock ranges are available,
@@ -1221,17 +1447,17 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
     clockRanges->minClock = pNv->MinVClockFreqKHz;
     clockRanges->maxClock = pNv->MaxVClockFreqKHz;
     clockRanges->clockIndex = -1;		/* programmable */
-    if(((pNv->Chipset & 0x0ff0) <= 0x0100) ||
-       ((pNv->Chipset & 0x0ff0) == 0x0150) ||
-       ((pNv->Chipset & 0x0ff0) >= 0x0300))
-    {
-       clockRanges->interlaceAllowed = TRUE;
-    } else {
-        /* No NV2x chips support interlaced modes and the only
-           NV1x chips that do are NV10 and NV15 */
-       clockRanges->interlaceAllowed = FALSE;
-    }
     clockRanges->doubleScanAllowed = TRUE;
+    if((pNv->Architecture == NV_ARCH_20) ||
+         ((pNv->Architecture == NV_ARCH_10) && 
+           ((pNv->Chipset & 0x0ff0) != 0x0100) &&
+           ((pNv->Chipset & 0x0ff0) != 0x0150)))
+    {
+       /* HW is broken */
+       clockRanges->interlaceAllowed = FALSE;
+    } else {
+       clockRanges->interlaceAllowed = TRUE;
+    }
 
     if(pNv->FlatPanel == 1) {
        clockRanges->interlaceAllowed = FALSE;
@@ -1307,42 +1533,42 @@ NVPreInit(ScrnInfoPtr pScrn, int flags)
      * section.
      */
 
-    if (xf86LoadSubModule(pScrn, "fb") == NULL) {
+    if (!(pMod = xf86LoadSubModule(pScrn, "fb"))) {
 	xf86FreeInt10(pNv->pInt);
 	NVFreeRec(pScrn);
 	return FALSE;
     }
 
-    xf86LoaderReqSymLists(fbSymbols, NULL);
+    xf86LoaderModReqSymLists(pMod, fbSymbols, NULL);
     
     /* Load XAA if needed */
     if (!pNv->NoAccel) {
-	if (!xf86LoadSubModule(pScrn, "xaa")) {
+	if (!(pMod = xf86LoadSubModule(pScrn, "xaa"))) {
 	    xf86FreeInt10(pNv->pInt);
 	    NVFreeRec(pScrn);
 	    return FALSE;
 	}
-	xf86LoaderReqSymLists(xaaSymbols, NULL);
+	xf86LoaderModReqSymLists(pMod, xaaSymbols, NULL);
     }
 
     /* Load ramdac if needed */
     if (pNv->HWCursor) {
-	if (!xf86LoadSubModule(pScrn, "ramdac")) {
+	if (!(pMod = xf86LoadSubModule(pScrn, "ramdac"))) {
 	    xf86FreeInt10(pNv->pInt);
 	    NVFreeRec(pScrn);
 	    return FALSE;
 	}
-	xf86LoaderReqSymLists(ramdacSymbols, NULL);
+	xf86LoaderModReqSymLists(pMod, ramdacSymbols, NULL);
     }
 
     /* Load shadowfb if needed */
     if (pNv->ShadowFB) {
-	if (!xf86LoadSubModule(pScrn, "shadowfb")) {
+	if (!(pMod = xf86LoadSubModule(pScrn, "shadowfb"))) {
 	    xf86FreeInt10(pNv->pInt);
 	    NVFreeRec(pScrn);
 	    return FALSE;
 	}
-	xf86LoaderReqSymLists(shadowSymbols, NULL);
+	xf86LoaderModReqSymLists(pMod, shadowSymbols, NULL);
     }
 
     pNv->CurrentLayout.bitsPerPixel = pScrn->bitsPerPixel;
@@ -1429,9 +1655,6 @@ NVModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
     NVPtr pNv = NVPTR(pScrn);
     NVRegPtr nvReg;
 
-    /* Initialise the ModeReg values */
-    if (!vgaHWInit(pScrn, mode))
-	return FALSE;
     pScrn->vtSema = TRUE;
 
     vgaReg = &hwp->ModeReg;
@@ -1484,17 +1707,21 @@ NVRestore(ScrnInfoPtr pScrn)
     NVPtr pNv = NVPTR(pScrn);
     NVRegPtr nvReg = &pNv->SavedReg;
 
+    NVLockUnlock(pNv, 0);
+
     if(pNv->twoHeads) {
         VGA_WR08(pNv->PCIO, 0x03D4, 0x44);
-        VGA_WR08(pNv->PCIO, 0x03D5, nvReg->crtcOwner);
+        VGA_WR08(pNv->PCIO, 0x03D5, pNv->CRTCnumber * 0x3);
         NVLockUnlock(pNv, 0);
     }
 
-    NVLockUnlock(pNv, 0);
-
     /* Only restore text mode fonts/text for the primary card */
     vgaHWProtect(pScrn, TRUE);
     NVDACRestore(pScrn, vgaReg, nvReg, pNv->Primary);
+    if(pNv->twoHeads) {
+        VGA_WR08(pNv->PCIO, 0x03D4, 0x44);
+        VGA_WR08(pNv->PCIO, 0x03D5, nvReg->crtcOwner);
+    }
     vgaHWProtect(pScrn, FALSE);
 }
 
@@ -1509,15 +1736,33 @@ static void NVBacklightEnable(NVPtr pNv,  Bool on)
        (pNv->Chipset == 0x10DE0329))
     {
        /* NV17,18,34 Apple iMac, iBook, PowerBook */
-       CARD32 tmp;
-       tmp = pNv->PMC[0x10F0/4] & 0x7FFFFFFF;
-       pNv->PMC[0x10F0/4] = tmp;
-       tmp = pNv->PCRTC0[0x081C/4] & 0xFFFFFFFC;
-       if(on)
-           tmp |= 0x1;
-       pNv->PCRTC0[0x081C/4] = tmp;
-    }
+      CARD32 tmp_pmc, tmp_pcrt;
+      tmp_pmc = pNv->PMC[0x10F0/4] & 0x7FFFFFFF;
+      tmp_pcrt = pNv->PCRTC0[0x081C/4] & 0xFFFFFFFC;
+      if(on) {
+          tmp_pmc |= (1 << 31);
+          tmp_pcrt |= 0x1;
+      }
+      pNv->PMC[0x10F0/4] = tmp_pmc;
+      pNv->PCRTC0[0x081C/4] = tmp_pcrt;
+    } 
 #endif
+    
+    if(pNv->LVDS) {
+       if(pNv->twoHeads && ((pNv->Chipset & 0x0ff0) != 0x0110)) {
+           pNv->PMC[0x130C/4] = on ? 3 : 7; 
+       }
+    } else {
+       CARD32 fpcontrol;
+
+       fpcontrol = pNv->PRAMDAC[0x0848/4] & 0xCfffffCC;
+
+       /* cut the TMDS output */
+       if(on) fpcontrol |= pNv->fpSyncs;
+       else fpcontrol |= 0x20000022;
+
+       pNv->PRAMDAC[0x0848/4] = fpcontrol;
+    }
 }
 
 static void
@@ -1527,6 +1772,8 @@ NVDPMSSetLCD(ScrnInfoPtr pScrn, int PowerManagementMode, int flags)
 
   if (!pScrn->vtSema) return;
 
+  vgaHWDPMSSet(pScrn, PowerManagementMode, flags);
+
   switch (PowerManagementMode) {
   case DPMSModeStandby:  /* HSync: Off, VSync: On */
   case DPMSModeSuspend:  /* HSync: On, VSync: Off */
@@ -1538,7 +1785,6 @@ NVDPMSSetLCD(ScrnInfoPtr pScrn, int PowerManagementMode, int flags)
   default:
     break;
   }
-  vgaHWDPMSSet(pScrn, PowerManagementMode, flags);
 }
 
 
@@ -1640,7 +1886,7 @@ NVScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
      * function.  If not, the visuals will need to be setup before calling
      * a fb ScreenInit() function and fixed up after.
      *
-     * For most PC hardware at depths >= 8, the defaults that cfb uses
+     * For most PC hardware at depths >= 8, the defaults that fb uses
      * are not appropriate.  In this driver, we fixup the visuals after.
      */
 
@@ -1775,6 +2021,9 @@ NVScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
                case 16:	refreshArea = NVRefreshArea16;	break;
                case 32:	refreshArea = NVRefreshArea32;	break;
 	   }
+           xf86DisableRandR();
+           xf86DrvMsg(pScrn->scrnIndex, X_INFO,
+                      "Driver rotation enabled, RandR disabled\n");
 	}
 
 	ShadowFBInit(pScreen, refreshArea);
@@ -1821,6 +2070,13 @@ NVSave(ScrnInfoPtr pScrn)
     vgaHWPtr pVga = VGAHWPTR(pScrn);
     vgaRegPtr vgaReg = &pVga->SavedReg;
 
+    NVLockUnlock(pNv, 0);
+    if(pNv->twoHeads) {
+        VGA_WR08(pNv->PCIO, 0x03D4, 0x44);
+        VGA_WR08(pNv->PCIO, 0x03D5, pNv->CRTCnumber * 0x3);
+        NVLockUnlock(pNv, 0);
+    }
+
     NVDACSave(pScrn, vgaReg, nvReg, pNv->Primary);
 }
 
