@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/savage/savage_driver.c,v 1.53 2004/11/26 13:45:03 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/savage/savage_driver.c,v 1.57 2006/03/16 16:50:13 dawes Exp $ */
 /*
  * vim: sw=4 ts=8 ai ic:
  *
@@ -17,7 +17,7 @@
 
 #include "globals.h"
 #define DPMS_SERVER
-#include "extensions/dpms.h"
+#include <X11/extensions/dpms.h>
 
 #include "xf86xv.h"
 
@@ -253,15 +253,12 @@ static const char *ramdacSymbols[] = {
 static const char *vbeSymbols[] = {
     "VBEInit",
     "vbeDoEDID",
-#if 0
     "vbeFree",
-#endif
     NULL
 };
 
 #ifdef XFree86LOADER
 static const char *vbeOptSymbols[] = {
-    "vbeModeInit",
     "VBESetVBEMode",
     "VBEGetVBEInfo",
     "VBEFreeVBEInfo",
@@ -336,7 +333,7 @@ static XF86ModuleVersionInfo SavageVersRec = {
 
 XF86ModuleData savageModuleData = { &SavageVersRec, SavageSetup, NULL };
 
-static pointer SavageSetup(pointer module, pointer opts, int *errmaj,
+static pointer SavageSetup(ModuleDescPtr module, pointer opts, int *errmaj,
 			   int *errmin)
 {
     static Bool setupDone = FALSE;
@@ -344,9 +341,10 @@ static pointer SavageSetup(pointer module, pointer opts, int *errmaj,
     if (!setupDone) {
 	setupDone = TRUE;
 	xf86AddDriver(&SAVAGE, module, 0);
-	LoaderRefSymLists(vgaHWSymbols, fbSymbols, ramdacSymbols, 
-			  xaaSymbols, shadowSymbols, vbeSymbols, vbeOptSymbols,
-			  int10Symbols, i2cSymbols, ddcSymbols, NULL);
+	LoaderModRefSymLists(module, vgaHWSymbols, fbSymbols, ramdacSymbols, 
+			     xaaSymbols, shadowSymbols, vbeSymbols,
+			     vbeOptSymbols, int10Symbols, i2cSymbols,
+			     ddcSymbols, NULL);
 	return (pointer) 1;
     } else {
 	if (errmaj)
@@ -593,12 +591,15 @@ static Bool SavageGetRec(ScrnInfoPtr pScrn)
 
 static void SavageFreeRec(ScrnInfoPtr pScrn)
 {
+    SavagePtr psav;
+
     TRACE(( "SavageFreeRec(%x)\n", pScrn->driverPrivate ));
-    if (!pScrn->driverPrivate)
+    if (!(psav = pScrn->driverPrivate))
 	return;
+    vbeFree(psav->pVbe);
+    SavageUnmapMem(pScrn, 1);
     xfree(pScrn->driverPrivate);
     pScrn->driverPrivate = NULL;
-    SavageUnmapMem(pScrn, 1);
 }
 
 
@@ -696,7 +697,8 @@ static Bool SavagePreInit(ScrnInfoPtr pScrn, int flags)
     int mclk;
     vgaHWPtr hwp;
     int vgaCRIndex, vgaCRReg;
-    pointer ddc;
+    ModuleDescPtr ddc;
+    ModuleDescPtr pMod;
 
     TRACE(("SavagePreInit(%d)\n", flags));
 
@@ -707,10 +709,10 @@ static Bool SavagePreInit(ScrnInfoPtr pScrn, int flags)
 	return TRUE;
     }
 
-    if (!xf86LoadSubModule(pScrn, "vgahw"))
+    if (!(pMod = xf86LoadSubModule(pScrn, "vgahw")))
 	return FALSE;
 
-    xf86LoaderReqSymLists(vgaHWSymbols, NULL);
+    xf86LoaderModReqSymLists(pMod, vgaHWSymbols, NULL);
     if (!vgaHWGetHWRec(pScrn))
 	return FALSE;
 
@@ -948,13 +950,13 @@ static Bool SavagePreInit(ScrnInfoPtr pScrn, int flags)
     }
     psav->EntityIndex = pEnt->index;
 
-    if (xf86LoadSubModule(pScrn, "int10")) {
- 	xf86LoaderReqSymLists(int10Symbols, NULL);
+    if ((pMod = xf86LoadSubModule(pScrn, "int10"))) {
+ 	xf86LoaderModReqSymLists(pMod, int10Symbols, NULL);
 	psav->pInt10 = xf86InitInt10(pEnt->index);
     }
 
-    if (xf86LoadSubModule(pScrn, "vbe")) {
-	xf86LoaderReqSymLists(vbeSymbols, NULL);
+    if ((pMod = xf86LoadVBEModule(pScrn))) {
+	xf86LoaderModReqSymLists(pMod, vbeSymbols, NULL);
 	psav->pVbe = VBEInit(psav->pInt10, pEnt->index);
     }
 
@@ -1011,7 +1013,6 @@ static Bool SavagePreInit(ScrnInfoPtr pScrn, int flags)
 
     if (!SavageMapMMIO(pScrn)) {
 	SavageFreeRec(pScrn);
-        vbeFree(psav->pVbe);
 	return FALSE;
     }
 
@@ -1041,7 +1042,6 @@ static Bool SavagePreInit(ScrnInfoPtr pScrn, int flags)
 	Gamma zeros = {0.0, 0.0, 0.0};
 
 	if (!xf86SetGamma(pScrn, zeros)) {
-	    vbeFree(psav->pVbe);
 	    SavageFreeRec(pScrn);
 	    return FALSE;
 	}
@@ -1221,7 +1221,7 @@ static Bool SavagePreInit(ScrnInfoPtr pScrn, int flags)
 	    xf86MonPtr pMon = NULL;
 #endif
 	   
-	    xf86LoaderReqSymLists(ddcSymbols, NULL);
+	    xf86LoaderModReqSymLists(ddc, ddcSymbols, NULL);
 #if 0
 /*
  * On many machines, the attempt to read DDC information via VBE puts the
@@ -1234,8 +1234,8 @@ static Bool SavagePreInit(ScrnInfoPtr pScrn, int flags)
 	    else 
 #endif
 	    if (!SavageDDC1(pScrn->scrnIndex)) {
-		if ( xf86LoadSubModule(pScrn, "i2c") ) {
-		    xf86LoaderReqSymLists(i2cSymbols,NULL);
+		if ((pMod = xf86LoadSubModule(pScrn, "i2c"))) {
+		    xf86LoaderModReqSymLists(pMod, i2cSymbols, NULL);
 		    if (SavageI2CInit(pScrn)) {
 			unsigned char tmp;
 
@@ -1383,7 +1383,6 @@ static Bool SavagePreInit(ScrnInfoPtr pScrn, int flags)
     if (i == -1) {
 	xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "xf86ValidateModes failure\n");
 	SavageFreeRec(pScrn);
-	vbeFree(psav->pVbe);
 	return FALSE;
     }
 
@@ -1392,7 +1391,6 @@ static Bool SavagePreInit(ScrnInfoPtr pScrn, int flags)
     if (i == 0 || pScrn->modes == NULL) {
 	xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "No valid modes found\n");
 	SavageFreeRec(pScrn);
-	vbeFree(psav->pVbe);
 	return FALSE;
     }
 
@@ -1443,39 +1441,35 @@ static Bool SavagePreInit(ScrnInfoPtr pScrn, int flags)
     xf86PrintModes(pScrn);
     xf86SetDpi(pScrn, 0, 0);
 
-    if (xf86LoadSubModule(pScrn, "fb") == NULL) {
+    if (!(pMod = xf86LoadSubModule(pScrn, "fb"))) {
 	SavageFreeRec(pScrn);
-	vbeFree(psav->pVbe);
 	return FALSE;
     }
 
-    xf86LoaderReqSymLists(fbSymbols, NULL);
+    xf86LoaderModReqSymLists(pMod, fbSymbols, NULL);
 
-    if( !psav->NoAccel ) {
-	if( !xf86LoadSubModule(pScrn, "xaa") ) {
+    if (!psav->NoAccel ) {
+	if (!(pMod = xf86LoadSubModule(pScrn, "xaa"))) {
 	    SavageFreeRec(pScrn);
-	    vbeFree(psav->pVbe);
 	    return FALSE;
 	}
-	xf86LoaderReqSymLists(xaaSymbols, NULL );
+	xf86LoaderModReqSymLists(pMod, xaaSymbols, NULL );
     }
 
     if (psav->hwcursor) {
-	if (!xf86LoadSubModule(pScrn, "ramdac")) {
+	if (!(pMod = xf86LoadSubModule(pScrn, "ramdac"))) {
 	    SavageFreeRec(pScrn);
-	    vbeFree(psav->pVbe);
 	    return FALSE;
 	}
-	xf86LoaderReqSymLists(ramdacSymbols, NULL);
+	xf86LoaderModReqSymLists(pMod, ramdacSymbols, NULL);
     }
 
     if (psav->shadowFB) {
-	if (!xf86LoadSubModule(pScrn, "shadowfb")) {
+	if (!(pMod = xf86LoadSubModule(pScrn, "shadowfb"))) {
 	    SavageFreeRec(pScrn);
-	    vbeFree(psav->pVbe);
 	    return FALSE;
 	}
-	xf86LoaderReqSymLists(shadowSymbols, NULL);
+	xf86LoaderModReqSymLists(pMod, shadowSymbols, NULL);
     }
     vbeFree(psav->pVbe);
 
@@ -3259,10 +3253,13 @@ static void
 SavageProbeDDC(ScrnInfoPtr pScrn, int index)
 {
     vbeInfoPtr pVbe;
-    if (xf86LoadSubModule(pScrn, "vbe")) {
+    ModuleDescPtr pMod;
+    if ((pMod = xf86LoadVBEModule(pScrn))) {
+	xf86LoaderModReqSymLists(pMod, vbeSymbols, NULL);
         pVbe = VBEInit(NULL,index);
         ConfiguredMonitor = vbeDoEDID(pVbe, NULL);
 	vbeFree(pVbe);
+	xf86UnloadSubModule(pMod);
     }
 }
 
