@@ -28,7 +28,7 @@
  *	    Massimiliano Ghilardi, max@Linuz.sns.it, some fixes to the
  *				   clockchip programming code.
  */
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/trident/trident_driver.c,v 1.193 2005/02/18 02:55:10 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/trident/trident_driver.c,v 1.199 2006/03/16 16:50:18 dawes Exp $ */
 
 #include "xf1bpp.h"
 #include "xf4bpp.h"
@@ -58,11 +58,11 @@
 #include "trident_regs.h"
 
 #define _XF86DGA_SERVER_
-#include "extensions/xf86dgastr.h"
+#include <X11/extensions/xf86dgastr.h>
 
 #include "globals.h"
 #define DPMS_SERVER
-#include "extensions/dpms.h"
+#include <X11/extensions/dpms.h>
 
 #include "xf86xv.h"
 
@@ -167,6 +167,7 @@ static SymTabRec TRIDENTChipsets[] = {
     { BLADEXP,			"bladeXP" },
     { CYBERBLADEXPAI1,		"cyberbladeXPAi1" },
     { CYBERBLADEXP4,		"cyberbladeXP4" },
+    { XP5,			"XP5" },
     { -1,				NULL }
 };
 
@@ -210,6 +211,7 @@ static PciChipsets TRIDENTPciChipsets[] = {
     { BLADEXP,		PCI_CHIP_9910,	RES_SHARED_VGA },
     { CYBERBLADEXPAI1,	PCI_CHIP_8820,	RES_SHARED_VGA },
     { CYBERBLADEXP4,	PCI_CHIP_2100,	RES_SHARED_VGA },
+    { XP5,		PCI_CHIP_2200,	RES_SHARED_VGA },
     { -1,		-1,		RES_UNDEFINED }
 };
     
@@ -309,6 +311,7 @@ static int ClockLimit[] = {
 	230000,
 	230000,
 	230000,
+	230000,
 };
 
 static int ClockLimit16bpp[] = {
@@ -352,6 +355,7 @@ static int ClockLimit16bpp[] = {
 	230000,
 	230000,
 	230000,
+	230000,
 }; 
 
 static int ClockLimit24bpp[] = {
@@ -395,6 +399,7 @@ static int ClockLimit24bpp[] = {
 	115000,
 	115000,
 	115000,
+	115000,
 };
 
 static int ClockLimit32bpp[] = {
@@ -439,6 +444,7 @@ static int ClockLimit32bpp[] = {
 	115000,
 	115000,
 	115000,
+	115000,
 };
 
 /*
@@ -577,16 +583,16 @@ static XF86ModuleVersionInfo tridentVersRec =
 XF86ModuleData tridentModuleData = { &tridentVersRec, tridentSetup, NULL };
 
 pointer
-tridentSetup(pointer module, pointer opts, int *errmaj, int *errmin)
+tridentSetup(ModuleDescPtr module, pointer opts, int *errmaj, int *errmin)
 {
     static Bool setupDone = FALSE;
 
     if (!setupDone) {
 	setupDone = TRUE;
 	xf86AddDriver(&TRIDENT, module, 0);
-	LoaderRefSymLists(vgahwSymbols, fbSymbols, i2cSymbols, vbeSymbols,
-			  miscfbSymbols, ramdacSymbols, int10Symbols,
-			  xaaSymbols, shadowSymbols, NULL);
+	LoaderModRefSymLists(module, vgahwSymbols, fbSymbols, i2cSymbols,
+			     vbeSymbols, miscfbSymbols, ramdacSymbols,
+			     int10Symbols, xaaSymbols, shadowSymbols, NULL);
 	return (pointer)TRUE;
     } 
 
@@ -1048,10 +1054,14 @@ static void
 TRIDENTProbeDDC(ScrnInfoPtr pScrn, int index)
 {
     vbeInfoPtr pVbe;
-    if (xf86LoadSubModule(pScrn, "vbe")) {
+    ModuleDescPtr pMod;
+
+    if ((pMod = xf86LoadVBEModule(pScrn))) {
+	xf86LoaderModReqSymLists(pMod, vbeSymbols, NULL);
 	pVbe = VBEInit(NULL,index);
 	ConfiguredMonitor = vbeDoEDID(pVbe, NULL);
 	vbeFree(pVbe);
+	xf86UnloadSubModule(pMod);
     }
 }
 
@@ -1075,6 +1085,7 @@ TRIDENTPreInit(ScrnInfoPtr pScrn, int flags)
     const char *Sym = "";
     Bool ddcLoaded = FALSE;
     char *s;
+    ModuleDescPtr pMod;
 
     /* Allocate the TRIDENTRec driverPrivate */
     if (!TRIDENTGetRec(pScrn)) {
@@ -1165,10 +1176,10 @@ TRIDENTPreInit(ScrnInfoPtr pScrn, int flags)
 	pix24bpp = xf86GetBppFromDepth(pScrn, 24);
 
     /* The vgahw module should be loaded here when needed */
-    if (!xf86LoadSubModule(pScrn, "vgahw"))
+    if (!(pMod = xf86LoadSubModule(pScrn, "vgahw")))
 	return FALSE;
 
-    xf86LoaderReqSymLists(vgahwSymbols, NULL);
+    xf86LoaderModReqSymLists(pMod, vgahwSymbols, NULL);
 
     /*
      * Allocate a vgaHWRec
@@ -1184,10 +1195,10 @@ TRIDENTPreInit(ScrnInfoPtr pScrn, int flags)
     xf86SetOperatingState(resVga, pTrident->pEnt->index, ResUnusedOpr);
 
     /* The ramdac module should be loaded here when needed */
-    if (!xf86LoadSubModule(pScrn, "ramdac"))
+    if (!(pMod = xf86LoadSubModule(pScrn, "ramdac")))
 	return FALSE;
 
-    xf86LoaderReqSymLists(ramdacSymbols, NULL);
+    xf86LoaderModReqSymLists(pMod, ramdacSymbols, NULL);
 
     /*
      * This must happen after pScrn->display has been set because
@@ -1530,29 +1541,21 @@ TRIDENTPreInit(ScrnInfoPtr pScrn, int flags)
      * when MMIO is turned on!
      */
 
-    if (xf86LoadSubModule(pScrn, "vbe")) {
+    if ((pMod = xf86LoadVBEModule(pScrn))) {
 	xf86MonPtr pMon;
 	vbeInfoPtr pVbe;
 
-        xf86LoaderReqSymLists(vbeSymbols, NULL);
-	pVbe =  VBEInit(NULL,pTrident->pEnt->index);
+        xf86LoaderModReqSymLists(pMod, vbeSymbols, NULL);
+	pVbe =  VBEInit(NULL, pTrident->pEnt->index);
 	pMon = vbeDoEDID(pVbe, NULL);
-#ifdef VBE_INFO
-	{
-	    VbeInfoBlock* vbeInfoBlockPtr;
-	    if ((vbeInfoBlockPtr = VBEGetVBEInfo(pVbe))) {
-		pTrident->vbeModes = VBEBuildVbeModeList(pVbe,vbeInfoBlockPtr);
-		VBEFreeVBEInfo(vbeInfoBlockPtr);
-	    }
-	}
-#endif
 	vbeFree(pVbe);
+	xf86UnloadSubModule(pMod);
 	if (pMon) {
-	    if (!xf86LoadSubModule(pScrn, "ddc")) {
+	    if (!(pMod = xf86LoadSubModule(pScrn, "ddc"))) {
 		TRIDENTFreeRec(pScrn);
 		return FALSE;
 	    } else {
-		xf86LoaderReqSymLists(ddcSymbols, NULL);
+		xf86LoaderModReqSymLists(pMod, ddcSymbols, NULL);
 		xf86SetDDCproperties(pScrn,xf86PrintEDID(pMon));
 		ddcLoaded = TRUE;
 	    }
@@ -2029,6 +2032,17 @@ TRIDENTPreInit(ScrnInfoPtr pScrn, int flags)
 	    chipset = "CyberBladeXP4";
 	    pTrident->NewClockCode = TRUE;
 	    pTrident->frequency = NTSC;
+	case XP5:
+    	    pTrident->ddc1Read = Tridentddc1Read;
+	    ramtype = "SGRAM";
+            pTrident->HasSGRAM = TRUE;
+	    pTrident->IsCyber = TRUE;
+	    pTrident->shadowNew = TRUE;
+	    pTrident->NoAccel = TRUE; /* for now */
+	    Support24bpp = TRUE;
+	    chipset = "XP5";
+	    pTrident->NewClockCode = TRUE;
+	    pTrident->frequency = NTSC;
 	    break;
     }
 
@@ -2075,6 +2089,27 @@ TRIDENTPreInit(ScrnInfoPtr pScrn, int flags)
 	pScrn->videoRam = pTrident->pEnt->device->videoRam;
 	from = X_CONFIG;
     } else {
+      if (pTrident->Chipset == XP5) {
+	OUTB(vgaIOBase + 4, 0x60);
+	videoram = INB(vgaIOBase + 5);
+	switch (videoram & 0x7) {
+ 	case 0x00:
+	    pScrn->videoRam = 131072;
+	    break;
+	case 0x01:
+	    pScrn->videoRam = 65536;
+	    break;
+	case 0x02:
+	    pScrn->videoRam = 32768;
+	    break;
+	case 0x03:
+	    pScrn->videoRam = 16384;
+	    break;
+	case 0x04:
+	    pScrn->videoRam = 8192;
+	    break;
+	}
+      } else
       if (pTrident->Chipset == CYBER9525DVD) {
 	pScrn->videoRam = 2560;
       } else
@@ -2388,7 +2423,7 @@ TRIDENTPreInit(ScrnInfoPtr pScrn, int flags)
 	break;
     }
 
-    if (mod && xf86LoadSubModule(pScrn, mod) == NULL) {
+    if (mod && !(pMod = xf86LoadSubModule(pScrn, mod))) {
 	if (IsPciCard && UseMMIO) {
     	    TRIDENTDisableMMIO(pScrn);
  	    TRIDENTUnmapMem(pScrn);
@@ -2399,13 +2434,13 @@ TRIDENTPreInit(ScrnInfoPtr pScrn, int flags)
 
     if (mod) {
 	if (Sym) {
-	    xf86LoaderReqSymbols(Sym, NULL);
+	    xf86LoaderModReqSymbols(pMod, Sym, NULL);
 	} else {
-	    xf86LoaderReqSymLists(fbSymbols, NULL);
+	    xf86LoaderModReqSymLists(pMod, fbSymbols, NULL);
 	}
     }
 
-    if (!xf86LoadSubModule(pScrn, "i2c")) {
+    if (!(pMod = xf86LoadSubModule(pScrn, "i2c"))) {
 	if (IsPciCard && UseMMIO) {
     	    TRIDENTDisableMMIO(pScrn);
  	    TRIDENTUnmapMem(pScrn);
@@ -2414,20 +2449,20 @@ TRIDENTPreInit(ScrnInfoPtr pScrn, int flags)
 	return FALSE;
     }
 
-    xf86LoaderReqSymLists(i2cSymbols, NULL);
+    xf86LoaderModReqSymLists(pMod, i2cSymbols, NULL);
 
     /* Load shadow if needed */
     if (pTrident->ShadowFB) {
-	if (!xf86LoadSubModule(pScrn, "shadow")) {
+	if (!(pMod = xf86LoadSubModule(pScrn, "shadow"))) {
 	    TRIDENTFreeRec(pScrn);
 	    return FALSE;
 	}
-	xf86LoaderReqSymLists(shadowSymbols, NULL);
+	xf86LoaderModReqSymLists(pMod, shadowSymbols, NULL);
     }
 
     /* Load XAA if needed */
     if (!pTrident->NoAccel) {
-	if (!xf86LoadSubModule(pScrn, "xaa")) {
+	if (!(pMod = xf86LoadSubModule(pScrn, "xaa"))) {
 	    if (IsPciCard && UseMMIO) {
     	    	TRIDENTDisableMMIO(pScrn);
  	    	TRIDENTUnmapMem(pScrn);
@@ -2436,7 +2471,7 @@ TRIDENTPreInit(ScrnInfoPtr pScrn, int flags)
 	    return FALSE;
 	}
 
-        xf86LoaderReqSymLists(xaaSymbols, NULL);
+        xf86LoaderModReqSymLists(pMod, xaaSymbols, NULL);
 
         switch (pScrn->displayWidth * pScrn->bitsPerPixel / 8) {
 	    case 512:
@@ -2459,7 +2494,7 @@ TRIDENTPreInit(ScrnInfoPtr pScrn, int flags)
     /* This gives us DDC1 - we should be able to get DDC2B using i2c */
 
     if (! ddcLoaded)
-	if (!xf86LoadSubModule(pScrn, "ddc")) {
+	if (!(pMod = xf86LoadSubModule(pScrn, "ddc"))) {
 	    if (IsPciCard && UseMMIO) {
 		TRIDENTDisableMMIO(pScrn);
 		TRIDENTUnmapMem(pScrn);
@@ -2468,7 +2503,7 @@ TRIDENTPreInit(ScrnInfoPtr pScrn, int flags)
 	    return FALSE;
 	}
     
-    xf86LoaderReqSymLists(ddcSymbols, NULL);
+    xf86LoaderModReqSymLists(pMod, ddcSymbols, NULL);
 
     if (IsPciCard && UseMMIO) {
         TRIDENTDisableMMIO(pScrn);
@@ -2606,7 +2641,7 @@ TRIDENTModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
     TRIDENTPtr pTrident = TRIDENTPTR(pScrn);
     TRIDENTRegPtr tridentReg;
 
-    WAITFORVSYNC;
+    if (!xf86IsPc98()) WAITFORVSYNC;
 
     TridentFindClock(pScrn,mode->Clock);
 
@@ -2632,6 +2667,7 @@ TRIDENTModeInit(ScrnInfoPtr pScrn, DisplayModePtr mode)
 	case BLADEXP:
 	case CYBERBLADEXPAI1:
 	case CYBERBLADEXP4:
+	case XP5:
 	    /* Get ready for MUX mode */
 	    if (pTrident->MUX && 
 		pScrn->bitsPerPixel == 8 && 
@@ -2753,6 +2789,7 @@ TRIDENTScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
     VisualPtr visual;
     unsigned char *FBStart;
     int width, height, displayWidth;
+    ModuleDescPtr pMod;
 
     /* 
      * First get the ScrnInfoRec
@@ -2769,20 +2806,10 @@ TRIDENTScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
     if (!TRIDENTMapMem(pScrn))
 	return FALSE;
 
-    if (!xf86IsPc98()) {
-#ifdef VBE_INFO
-	if (pTrident->vbeModes) {
-	    pTrident->pVbe = VBEInit(NULL,pTrident->pEnt->index);
-	    pTrident->Int10 = pTrident->pVbe->pInt10;
-	} else
-#endif
-	{
-	    if (xf86LoadSubModule(pScrn, "int10")) {
-		xf86LoaderReqSymLists(int10Symbols, NULL);
-		xf86DrvMsg(pScrn->scrnIndex,X_INFO,"Initializing int10\n");
-		pTrident->Int10 = xf86InitInt10(pTrident->pEnt->index);
-	    }
-	}
+    if (!xf86IsPc98() && (pMod = xf86LoadSubModule(pScrn, "int10"))) {
+	xf86LoaderModReqSymLists(pMod, int10Symbols, NULL);
+	xf86DrvMsg(pScrn->scrnIndex,X_INFO,"Initializing int10\n");
+	pTrident->Int10 = xf86InitInt10(pTrident->pEnt->index);
     }
     
     hwp = VGAHWPTR(pScrn);
@@ -2821,7 +2848,7 @@ TRIDENTScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
      * function.  If not, the visuals will need to be setup before calling
      * a fb ScreenInit() function and fixed up after.
      *
-     * For most PC hardware at depths >= 8, the defaults that cfb uses
+     * For most PC hardware at depths >= 8, the defaults that fb uses
      * are not appropriate.  In this driver, we fixup the visuals after.
      */
 
@@ -3635,26 +3662,6 @@ tridentSetModeBIOS(ScrnInfoPtr pScrn, DisplayModePtr mode)
 {
     TRIDENTPtr pTrident = TRIDENTPTR(pScrn);
 
-
-#ifdef VBE_INFO
-    if (pTrident->vbeModes) {
-	vbeSaveRestoreRec vbesr;
-	vbesr.stateMode = VBECalcVbeModeIndex(pTrident->vbeModes,
-					     mode, pScrn->bitsPerPixel);
-	vbesr.pstate = NULL;
-	if (vbesr.stateMode) {
-	    if (IsPciCard && UseMMIO) 
-		TRIDENTDisableMMIO(pScrn);
-	    VBEVesaSaveRestore(pTrident->pVbe,&vbesr,MODE_RESTORE);
-	    if (IsPciCard && UseMMIO) 
-		TRIDENTEnableMMIO(pScrn);
-	    return;
-	} else
-	    xf86DrvMsg(pScrn->scrnIndex,X_WARNING,"No BIOS Mode matches "
-		       "%ix%I@%ibpp\n",mode->HDisplay,mode->VDisplay,
-		       pScrn->bitsPerPixel);
-    } 
-#endif
     /* This function is only for LCD screens, and also when we have
      * int10 available */
 
