@@ -1,4 +1,5 @@
-/* $XConsortium: xf86Elo.c /main/13 1996/10/25 14:11:31 kaleb $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/input/elographics/xf86Elo.c,v 1.22 2006/03/16 16:50:28 dawes Exp $ */
+
 /*
  * Copyright 1995, 1999 by Patrick Lecoanet, France. <lecoanet@cena.dgac.fr>
  *                                                                            
@@ -22,8 +23,6 @@
  *
  */
 
-/* $XFree86: xc/programs/Xserver/hw/xfree86/input/elographics/xf86Elo.c,v 1.20 2004/04/26 22:48:21 dawes Exp $ */
-
 /*
  *******************************************************************************
  *******************************************************************************
@@ -1171,8 +1170,10 @@ InputDriverRec ELOGRAPHICS = {
 };
 
 #ifdef XFree86LOADER
+static MODULESETUPPROTO(Plug);
+
 static pointer
-Plug(pointer	module,
+Plug(ModuleDescPtr	module,
      pointer	options,
      int	*errmaj,
      int	*errmin)
