@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/loader/loadmod.c,v 1.74 2004/02/13 23:58:45 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/loader/loadmod.c,v 1.79 2006/03/26 02:25:08 dawes Exp $ */
 
 /*
  *
@@ -23,7 +23,7 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 /*
- * Copyright (c) 1997-2002 by The XFree86 Project, Inc.
+ * Copyright (c) 1997-2006 by The XFree86 Project, Inc.
  * All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining
@@ -68,6 +68,51 @@
  * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
+/*
+ * Copyright 2003-2006 by David H. Dawes.
+ * Copyright 2003-2006 by X-Oz Technologies.
+ * All rights reserved.
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ * 
+ *  1. Redistributions of source code must retain the above copyright
+ *     notice, this list of conditions, and the following disclaimer.
+ *
+ *  2. Redistributions in binary form must reproduce the above
+ *     copyright notice, this list of conditions and the following
+ *     disclaimer in the documentation and/or other materials provided
+ *     with the distribution.
+ * 
+ *  3. The end-user documentation included with the redistribution,
+ *     if any, must include the following acknowledgment: "This product
+ *     includes software developed by X-Oz Technologies
+ *     (http://www.x-oz.com/)."  Alternately, this acknowledgment may
+ *     appear in the software itself, if and wherever such third-party
+ *     acknowledgments normally appear.
+ *
+ *  4. Except as contained in this notice, the name of X-Oz
+ *     Technologies shall not be used in advertising or otherwise to
+ *     promote the sale, use or other dealings in this Software without
+ *     prior written authorization from X-Oz Technologies.
+ *
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR
+ * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL X-OZ TECHNOLOGIES OR ITS CONTRIBUTORS
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * 
+ */
 
 #include "os.h"
 /* For stat() and related stuff */
@@ -81,6 +126,10 @@
 #ifdef XINPUT
 #include "xf86Xinput.h"
 #endif
+
+#ifndef LOADERDEBUG
+#define LOADERDEBUG 0
+#endif
 #include "loader.h"
 #include "xf86Optrec.h"
 
@@ -89,18 +138,12 @@
 #include <dirent.h>
 #include <limits.h>
 
-extern int check_unresolved_sema;
-
 typedef struct _pattern {
     const char *pattern;
     regex_t rex;
 } PatternRec, *PatternPtr;
 
-/* Prototypes for static functions */
-static char *FindModule(const char *, const char *, const char **,
-			PatternPtr);
-static Bool CheckVersion(const char *, XF86ModuleVersionInfo *,
-			 const XF86ModReqInfo *);
+/* Forward prototypes for static functions */
 static void UnloadModuleOrDriver(ModuleDescPtr mod);
 static char *LoaderGetCanonicalName(const char *, PatternPtr);
 static void RemoveChild(ModuleDescPtr);
@@ -114,14 +157,33 @@ ModuleVersions LoaderVersionInfo = {
     ABI_FONT_VERSION
 };
 
-#if 0
-void
-LoaderFixups(void)
+#if LOADERDEBUG
+static void
+PrintDiagnostics(const char *module, const char *what)
 {
-    /* Need to call LRS here because the frame buffers get loaded last,
-     * and the drivers depend on them. */
+#ifdef linux
+    char procdev[] = "/proc/XXXXX/maps";
+    FILE *f = NULL;
+    char buf[1024];
+#endif
 
-    LoaderResolveSymbols();
+    if (!(LoaderDebugLevel & LOADER_DEBUG_DIAGNOSTICS))
+	return;
+
+    LoaderDebugMsg(LOADER_DEBUG_DIAGNOSTICS,
+		   "LoaderDiagnostics for %s of module %s\n", what, module);
+#ifdef linux
+    sprintf(procdev, "/proc/%d/maps", getpid());
+    f = fopen(procdev, "r");
+    if (f) {
+	while (fgets(buf, sizeof(buf), f))
+	    LoaderDebugMsg(LOADER_DEBUG_DIAGNOSTICS, "%s", buf);
+	fclose(f);
+    } else {
+	LoaderDebugMsg(LOADER_DEBUG_DIAGNOSTICS,
+		       "Cannot open %s: %s\n", procdev, strerror(errno));
+    }
+#endif
 }
 #endif
 
@@ -595,6 +657,89 @@ LoaderFreeDirList(char **list)
     FreeStringList(list);
 }
 
+static Bool
+CheckRequirements(const XF86ModuleVersionInfo * data,
+		  const XF86ModReqInfo * req)
+{
+    if (!req)
+	return TRUE;
+
+    if (!data) {
+	xf86MsgVerb(X_WARNING, 2, "No version information to verify\n");
+	return FALSE;
+    }
+
+    if (req->majorversion != MAJOR_UNSPEC) {
+	if (data->majorversion != req->majorversion) {
+	    xf86MsgVerb(X_WARNING, 2, "Module major version (%d) doesn't match"
+			" required major version (%d)\n",
+			data->majorversion, req->majorversion);
+	    return FALSE;
+	}
+
+	if (req->minorversion != MINOR_UNSPEC) {
+	    if (data->minorversion < req->minorversion) {
+		xf86MsgVerb(X_WARNING, 2, "Module minor version (%d) is less"
+			    " than the required minor version (%d)\n",
+			    data->minorversion, req->minorversion);
+		return FALSE;
+	    }
+
+	    if ((data->minorversion == req->minorversion) &&
+		(req->patchlevel != PATCH_UNSPEC) &&
+		(data->patchlevel < req->patchlevel)) {
+		xf86MsgVerb(X_WARNING, 2, "Module patch level (%d) is less"
+			    " than the required patch level (%d)\n",
+			    data->patchlevel, req->patchlevel);
+		return FALSE;
+	    }
+	}
+    }
+
+    if (req->moduleclass) {
+	if (!data->moduleclass ||
+	    strcmp(req->moduleclass, data->moduleclass)) {
+	    xf86MsgVerb(X_WARNING, 2, "Module class (%s) doesn't match the"
+			" required class (%s)\n",
+			data->moduleclass ? data->moduleclass : "<NONE>",
+			req->moduleclass);
+	    return FALSE;
+	}
+    } else if (req->abiclass != ABI_CLASS_NONE) {
+	if (!data->abiclass || strcmp(req->abiclass, data->abiclass)) {
+	    xf86MsgVerb(X_WARNING, 2, "ABI class (%s) doesn't match the"
+			" required ABI class (%s)\n",
+			data->abiclass ? data->abiclass : "<NONE>",
+			req->abiclass);
+	    return FALSE;
+	}
+    }
+
+    if ((req->abiclass != ABI_CLASS_NONE) &&
+	(req->abiversion != ABI_VERS_UNSPEC)) {
+	int reqmaj, reqmin, maj, min;
+
+	reqmaj = GET_ABI_MAJOR(req->abiversion);
+	maj = GET_ABI_MAJOR(data->abiversion);
+	if (maj != reqmaj) {
+	    xf86MsgVerb(X_WARNING, 2, "ABI major version (%d) doesn't match"
+			" the required ABI major version (%d)\n",
+			maj, reqmaj);
+	    return FALSE;
+	}
+
+	reqmin = GET_ABI_MINOR(req->abiversion);
+	min = GET_ABI_MINOR(data->abiversion);
+	if (min < reqmin) {
+	    xf86MsgVerb(X_WARNING, 2, "Module ABI minor version (%d) is older"
+			" than that required (%d)\n", min, reqmin);
+	    return FALSE;
+	}
+    }
+
+    return TRUE;
+}
+
 static Bool
 CheckVersion(const char *module, XF86ModuleVersionInfo * data,
 	     const XF86ModReqInfo * req)
@@ -691,70 +836,9 @@ CheckVersion(const char *module, XF86ModuleVersionInfo * data,
     }
 
     /* Check against requirements that the caller has specified */
-    if (req) {
-	if (req->majorversion != MAJOR_UNSPEC) {
-	    if (data->majorversion != req->majorversion) {
-		xf86MsgVerb(X_WARNING, 2, "module major version (%d) "
-			    "doesn't match required major version (%d)\n",
-			    data->majorversion, req->majorversion);
-		return FALSE;
-	    } else if (req->minorversion != MINOR_UNSPEC) {
-		if (data->minorversion < req->minorversion) {
-		    xf86MsgVerb(X_WARNING, 2, "module minor version (%d) "
-				"is less than the required minor version (%d)\n",
-				data->minorversion, req->minorversion);
-		    return FALSE;
-		} else if (data->minorversion == req->minorversion &&
-			   req->patchlevel != PATCH_UNSPEC) {
-		    if (data->patchlevel < req->patchlevel) {
-			xf86MsgVerb(X_WARNING, 2, "module patch level (%d) "
-				    "is less than the required patch level (%d)\n",
-				    data->patchlevel, req->patchlevel);
-			return FALSE;
-		    }
-		}
-	    }
-	}
-	if (req->moduleclass) {
-	    if (!data->moduleclass ||
-		strcmp(req->moduleclass, data->moduleclass)) {
-		xf86MsgVerb(X_WARNING, 2, "Module class (%s) doesn't match "
-			    "the required class (%s)\n",
-			    data->moduleclass ? data->moduleclass : "<NONE>",
-			    req->moduleclass);
-		return FALSE;
-	    }
-	} else if (req->abiclass != ABI_CLASS_NONE) {
-	    if (!data->abiclass || strcmp(req->abiclass, data->moduleclass)) {
-		xf86MsgVerb(X_WARNING, 2, "ABI class (%s) doesn't match the "
-			    "required ABI class (%s)\n",
-			    data->abiclass ? data->abiclass : "<NONE>",
-			    req->abiclass);
-		return FALSE;
-	    }
-	}
-	if ((req->abiclass != ABI_CLASS_NONE) &&
-	    req->abiversion != ABI_VERS_UNSPEC) {
-	    int reqmaj, reqmin, maj, min;
+    if (!CheckRequirements(data, req))
+	return FALSE;
 
-	    reqmaj = GET_ABI_MAJOR(req->abiversion);
-	    reqmin = GET_ABI_MINOR(req->abiversion);
-	    maj = GET_ABI_MAJOR(data->abiversion);
-	    min = GET_ABI_MINOR(data->abiversion);
-	    if (maj != reqmaj) {
-		xf86MsgVerb(X_WARNING, 2, "ABI major version (%d) doesn't "
-			    "match the required ABI major version (%d)\n",
-			    maj, reqmaj);
-		return FALSE;
-	    }
-	    /* XXX Maybe this should be the other way around? */
-	    if (min > reqmin) {
-		xf86MsgVerb(X_WARNING, 2, "module ABI minor version (%d) "
-			    "is new than that available (%d)\n", min, reqmin);
-		return FALSE;
-	    }
-	}
-    }
 #ifdef NOTYET
     if (data->checksum) {
 	/* verify the checksum field */
@@ -766,6 +850,12 @@ CheckVersion(const char *module, XF86ModuleVersionInfo * data,
     return TRUE;
 }
 
+void
+LoaderSetParentModuleRequirements(ModuleDescPtr mod, XF86ModReqInfo *req)
+{
+    mod->ParentReq = req;
+}
+
 ModuleDescPtr
 LoadSubModule(ModuleDescPtr parent, const char *module,
 	      const char **subdirlist, const char **patternlist,
@@ -796,8 +886,24 @@ LoadSubModule(ModuleDescPtr parent, const char *module,
     submod = LoadModule(module, NULL, subdirlist, patternlist, options,
 			modreq, errmaj, errmin);
     if (submod) {
-	parent->child = AddSibling(parent->child, submod);
 	submod->parent = parent;
+
+	/* Check requirements against parent modules */
+	while (parent) {
+	    if (!CheckRequirements(parent->VersionInfo, submod->ParentReq)) {
+		UnloadModule(submod);
+		if (errmaj)
+		    *errmaj = LDR_MISMATCH;
+		if (errmin)
+		    *errmin = 0;
+		return NULL;
+	    }
+
+	    parent = parent->parent;
+	}
+
+	parent = submod->parent;
+	parent->child = AddSibling(parent->child, submod);
     }
     return submod;
 }
@@ -830,6 +936,9 @@ DuplicateModule(ModuleDescPtr mod, ModuleDescPtr parent)
     ret->sib = DuplicateModule(mod->sib, parent);
     ret->parent = parent;
     ret->VersionInfo = mod->VersionInfo;
+    ret->ParentReq = mod->ParentReq;
+
+    DuplicateSymbolLists(mod, ret);
 
     return ret;
 }
@@ -838,7 +947,7 @@ DuplicateModule(ModuleDescPtr mod, ModuleDescPtr parent)
  * LoadModule: load a module
  *
  * module       The module name.  Normally this is not a filename but the
- *              module's "canonical name.  A full pathname is, however,
+ *              module's "canonical" name.  A full pathname is, however,
  *              also accepted.
  * path         A comma separated list of module directories.
  * subdirlist   A NULL terminated list of subdirectories to search.  When
@@ -968,27 +1077,24 @@ LoadModule(const char *module, const char *path, const char **subdirlist,
 	    *errmin = 0;
 	goto LoadModule_fail;
     }
-    ret->handle = LoaderOpen(found, name, 0, errmaj, errmin, &wasLoaded);
+    ret->handle = LoaderOpen(found, name, 0, errmaj, errmin, &wasLoaded,
+			     (char **)&initdata);
     if (ret->handle < 0)
 	goto LoadModule_fail;
 
     ret->filename = xstrdup(found);
 
-    /*
-     * now check if the special data object <modulename>ModuleData is
-     * present.
-     */
-    p = xalloc(strlen(name) + strlen("ModuleData") + 1);
-    if (!p) {
-	if (errmaj)
-	    *errmaj = LDR_NOMEM;
-	if (errmin)
-	    *errmin = 0;
-	goto LoadModule_fail;
+    /* This is needed for dlopen modules. */
+    if (!initdata) {
+	char *md;
+
+	xasprintf(&md, "%s" MODULE_DATA_NAME, name);
+	if (md) {
+	    initdata = LoaderSymbol(md);
+	    xfree(md);
+	}
     }
-    strcpy(p, name);
-    strcat(p, "ModuleData");
-    initdata = LoaderSymbol(p);
+
     if (initdata) {
 	ModuleSetupProc setup;
 	ModuleTearDownProc teardown;
@@ -998,25 +1104,27 @@ LoadModule(const char *module, const char *path, const char **subdirlist,
 	setup = initdata->setup;
 	teardown = initdata->teardown;
 
-	if (!wasLoaded) {
-	    if (vers) {
-		if (!CheckVersion(module, vers, modreq)) {
-		    if (errmaj)
-			*errmaj = LDR_MISMATCH;
-		    if (errmin)
-			*errmin = 0;
-		    goto LoadModule_fail;
-		}
-	    } else {
-		xf86Msg(X_ERROR,
-			"LoadModule: Module %s does not supply"
-			" version information\n", module);
+	/*
+	 * Different loads of the same module may have different versioning
+	 * requirements, so always check.
+	 */
+	if (vers) {
+	    if (!CheckVersion(module, vers, modreq)) {
 		if (errmaj)
-		    *errmaj = LDR_INVALID;
+		    *errmaj = LDR_MISMATCH;
 		if (errmin)
 		    *errmin = 0;
 		goto LoadModule_fail;
 	    }
+	} else {
+	    xf86Msg(X_ERROR,
+		    "LoadModule: Module %s does not supply version"
+		    " information\n", module);
+	    if (errmaj)
+		*errmaj = LDR_INVALID;
+	    if (errmin)
+		*errmin = 0;
+	    goto LoadModule_fail;
 	}
 	if (setup)
 	    ret->SetupProc = setup;
@@ -1030,8 +1138,9 @@ LoadModule(const char *module, const char *path, const char **subdirlist,
 	    goto LoadModule_exit;
 
 	/* no initdata, fail the load */
-	xf86Msg(X_ERROR, "LoadModule: Module %s does not have a %s "
-		"data object.\n", module, p);
+	xf86Msg(X_ERROR, "LoadModule: Module %s does not have a %s"
+			 MODULE_DATA_NAME " data object.\n",
+		module, name);
 	if (errmaj)
 	    *errmaj = LDR_INVALID;
 	if (errmin)
@@ -1047,6 +1156,9 @@ LoadModule(const char *module, const char *path, const char **subdirlist,
 	xf86Msg(X_WARNING, "Module Options present, but no SetupProc "
 		"available for %s\n", module);
     }
+#if LOADERDEBUG
+    PrintDiagnostics(module, "load");
+#endif
     goto LoadModule_exit;
 
   LoadModule_fail:
@@ -1102,7 +1214,7 @@ UnloadModuleOrDriver(ModuleDescPtr mod)
 
     if ((mod->TearDownProc) && (mod->TearDownData))
 	mod->TearDownProc(mod->TearDownData);
-    LoaderUnload(mod->handle);
+    LoaderUnload(mod);
 
     if (mod->child)
 	UnloadModuleOrDriver(mod->child);
@@ -1126,7 +1238,7 @@ UnloadSubModule(ModuleDescPtr mod)
 
     if ((mod->TearDownProc) && (mod->TearDownData))
 	mod->TearDownProc(mod->TearDownData);
-    LoaderUnload(mod->handle);
+    LoaderUnload(mod);
 
     RemoveChild(mod);
 
@@ -1169,7 +1281,6 @@ NewModuleDesc(const char *name)
 	mdp->child = NULL;
 	mdp->sib = NULL;
 	mdp->parent = NULL;
-	mdp->demand_next = NULL;
 	mdp->name = xstrdup(name);
 	mdp->filename = NULL;
 	mdp->identifier = NULL;
@@ -1179,6 +1290,7 @@ NewModuleDesc(const char *name)
 	mdp->SetupProc = NULL;
 	mdp->TearDownProc = NULL;
 	mdp->TearDownData = NULL;
+	mdp->ParentReq = NULL;
     }
 
     return (mdp);
@@ -1274,7 +1386,7 @@ LoaderErrorMsg(const char *name, const char *modname, int errmaj, int errmin)
 	msg = "module-specific error";
 	break;
     default:
-	msg = "uknown error";
+	msg = "unknown error";
     }
     if (name)
 	xf86Msg(X_ERROR, "%s: Failed to load module \"%s\" (%s, %d)\n",
@@ -1330,3 +1442,19 @@ LoaderGetModuleVersion(ModuleDescPtr mod)
 				  mod->VersionInfo->minorversion,
 				  mod->VersionInfo->patchlevel);
 }
+
+ModuleDescPtr
+LoaderGetSubModuleByName(ModuleDescPtr mod, const char *name)
+{
+    ModuleDescPtr m;
+
+    if (!mod || !name)
+	return NULL;
+
+    for (m = mod->child; m; m = m->sib) {
+	if (!strcmp(m->name, name))
+	    return m;
+    }
+    return NULL;
+}
+
