@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/mi/miarc.c,v 3.15 2003/12/06 18:46:28 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/mi/miarc.c,v 3.20 2006/02/20 00:14:38 dawes Exp $ */
 /***********************************************************
 
 Copyright 1987, 1998  The Open Group
@@ -45,7 +45,7 @@ ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 SOFTWARE.
 
 ******************************************************************/
-/* $Xorg: miarc.c,v 1.4 2001/02/09 02:05:20 xorgcvs Exp $ */
+
 /* Author: Keith Packard and Bob Scheifler */
 /* Warning: this code is toxic, do not dally very long here. */
 
@@ -56,8 +56,8 @@ SOFTWARE.
 #include <math.h>
 #undef _XOPEN_SOURCE
 #endif
-#include "X.h"
-#include "Xprotostr.h"
+#include <X11/X.h>
+#include <X11/Xprotostr.h>
 #include "misc.h"
 #include "gcstruct.h"
 #include "scrnintstr.h"
@@ -66,13 +66,17 @@ SOFTWARE.
 #include "mifpoly.h"
 #include "mi.h"
 #include "mifillarc.h"
-#include "Xfuncproto.h"
+#include <X11/Xfuncproto.h>
 
 static double miDsin(double a);
 static double miDcos(double a);
 static double miDasin(double v);
 static double miDatan2(double dy, double dx);
+#if !defined(__USE_XOPEN_EXTENDED) || !__USE_XOPEN_EXTENDED
+#if !defined(sun)
 double	cbrt(double);
+#endif
+#endif
 
 #ifdef ICEILTEMPDECL
 ICEILTEMPDECL
@@ -106,12 +110,14 @@ ICEILTEMPDECL
 #endif
 
 #ifdef USE_INLINE
-inline static const int max (const int x, const int y)
+inline static int
+max(const int x, const int y)
 {
 	return x>y? x:y;
 }
 
-inline static const int min (const int x, const int y)
+inline static int
+min (const int x, const int y)
 {
 	return x<y? x:y;
 }
@@ -119,13 +125,13 @@ inline static const int min (const int x, const int y)
 #else
 
 static int
-max (int x, int y)
+max(int x, int y)
 {
 	return x>y? x:y;
 }
 
 static int
-min (int x, int y)
+min(int x, int y)
 {
 	return x<y? x:y;
 }
@@ -243,7 +249,7 @@ typedef struct _miPolyArc {
 static CARD32 gcvals[6];
 
 static void fillSpans(DrawablePtr pDrawable, GCPtr pGC);
-static void newFinalSpan(int y, register int xmin, register int xmax);
+static void newFinalSpan(int y, int xmin, int xmax);
 static void drawArc(xArc *tarc, int l, int a0, int a1, miArcFacePtr right,
 		    miArcFacePtr left);
 static void drawZeroArc(DrawablePtr pDraw, GCPtr pGC, xArc *tarc, int lw,
@@ -428,9 +434,7 @@ static RESTYPE cacheType;
  */
 /*ARGSUSED*/
 int
-miFreeArcCache (data, id)
-    pointer	    data;
-    XID		    id;
+miFreeArcCache(pointer data, XID id)
 {
     int k;
     arcCacheRec *cent;
@@ -458,11 +462,11 @@ miComputeCircleSpans(
     xArc *parc,
     miArcSpanData *spdata)
 {
-    register miArcSpan *span;
+    miArcSpan *span;
     int doinner;
-    register int x, y, e;
+    int x, y, e;
     int xk, yk, xm, ym, dx, dy;
-    register int slw, inslw;
+    int slw, inslw;
     int inx = 0, iny, ine = 0;
     int inxk = 0, inyk = 0, inxm = 0, inym = 0;
 
@@ -526,7 +530,7 @@ miComputeEllipseSpans(
     xArc *parc,
     miArcSpanData *spdata)
 {
-    register miArcSpan *span;
+    miArcSpan *span;
     double w, h, r, xorg;
     double Hs, Hf, WH, K, Vk, Nk, Fk, Vr, N, Nc, Z, rs;
     double A, T, b, d, x, y, t, inx, outx = 0.0, hepp, hepm;
@@ -857,12 +861,12 @@ tailX(
 static miArcSpanData *
 miComputeWideEllipse(
     int		   lw,
-    register xArc *parc,
+    xArc *parc,
     Bool	  *mustFree)
 {
-    register miArcSpanData *spdata;
-    register arcCacheRec *cent, *lruent;
-    register int k;
+    miArcSpanData *spdata;
+    arcCacheRec *cent, *lruent;
+    int k;
     arcCacheRec fakeent;
 
     if (!lw)
@@ -940,14 +944,14 @@ miFillWideEllipse(
     xArc	*parc)
 {
     DDXPointPtr points;
-    register DDXPointPtr pts;
+    DDXPointPtr pts;
     int *widths;
-    register int *wids;
+    int *wids;
     miArcSpanData *spdata;
     Bool mustFree;
-    register miArcSpan *span;
-    register int xorg, yorgu, yorgl;
-    register int n;
+    miArcSpan *span;
+    int xorg, yorgu, yorgl;
+    int n;
 
     yorgu = parc->height + pGC->lineWidth;
     n = (sizeof(int) * 2) * yorgu;
@@ -1068,13 +1072,9 @@ miFillWideEllipse(
  */
 
 void
-miPolyArc(pDraw, pGC, narcs, parcs)
-    DrawablePtr	pDraw;
-    GCPtr	pGC;
-    int		narcs;
-    xArc	*parcs;
+miPolyArc(DrawablePtr pDraw, GCPtr pGC, int narcs, xArc *parcs)
 {
-    register int		i;
+    int		i;
     xArc			*parc;
     int				xMin, xMax, yMin, yMax;
     int				pixmapWidth = 0, pixmapHeight = 0;
@@ -1297,7 +1297,7 @@ miPolyArc(pDraw, pGC, narcs, parcs)
 }
 
 static double
-angleBetween (SppPointRec center, SppPointRec point1, SppPointRec point2)
+angleBetween(SppPointRec center, SppPointRec point1, SppPointRec point2)
 {
 	double	a1, a2, a;
 	
@@ -3136,9 +3136,9 @@ static struct finalSpanChunk	*chunks;
 struct finalSpan *
 realAllocSpan ()
 {
-	register struct finalSpanChunk	*newChunk;
-	register struct finalSpan	*span;
-	register int			i;
+	struct finalSpanChunk	*newChunk;
+	struct finalSpan	*span;
+	int			i;
 
 	newChunk = (struct finalSpanChunk *) xalloc (sizeof (struct finalSpanChunk));
 	if (!newChunk)
@@ -3176,12 +3176,12 @@ fillSpans (
     DrawablePtr	pDrawable,
     GCPtr	pGC)
 {
-	register struct finalSpan	*span;
-	register DDXPointPtr		xSpan;
-	register int			*xWidth;
-	register int			i;
-	register struct finalSpan	**f;
-	register int			spany;
+	struct finalSpan	*span;
+	DDXPointPtr		xSpan;
+	int			*xWidth;
+	int			i;
+	struct finalSpan	**f;
+	int			spany;
 	DDXPointPtr			xSpans;
 	int				*xWidths;
 
@@ -3277,11 +3277,11 @@ realFindSpan (int y)
 static void
 newFinalSpan (
     int		y,
-    register int	xmin,
-    register int	xmax)
+    int	xmin,
+    int	xmax)
 {
-	register struct finalSpan	*x;
-	register struct finalSpan	**f;
+	struct finalSpan	*x;
+	struct finalSpan	**f;
 	struct finalSpan		*oldx;
 	struct finalSpan		*prev;
 
