@@ -1,4 +1,3 @@
-/* $Xorg: connection.c,v 1.6 2001/02/09 02:05:23 xorgcvs Exp $ */
 /***********************************************************
 
 Copyright 1987, 1989, 1998  The Open Group
@@ -45,7 +44,7 @@ ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 SOFTWARE.
 
 ******************************************************************/
-/* $XFree86: xc/programs/Xserver/os/connection.c,v 3.67 2004/06/23 19:40:17 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/os/connection.c,v 3.70 2006/03/06 16:06:23 dawes Exp $ */
 /*****************************************************************
  *  Stuff to create connections --- OS dependent
  *
@@ -66,8 +65,8 @@ SOFTWARE.
 #ifdef WIN32
 #include <X11/Xwinsock.h>
 #endif
-#include "X.h"
-#include "Xproto.h"
+#include <X11/X.h>
+#include <X11/Xproto.h>
 #include <X11/Xtrans.h>
 #include <errno.h>
 #include <signal.h>
@@ -141,11 +140,11 @@ extern __const__ int _nfiles;
 #include "opaque.h"
 #include "dixstruct.h"
 #ifdef XAPPGROUP
-#include "extensions/Xagsrv.h"
+#include <X11/extensions/Xagsrv.h>
 #endif
 #ifdef XCSECURITY
 #define _SECURITY_SERVER
-#include "extensions/security.h"
+#include <X11/extensions/security.h>
 #endif
 #ifdef LBX
 #include "colormapst.h"
@@ -352,7 +351,7 @@ CreateWellKnownSockets(void)
     OsSignal (SIGPIPE, SIG_IGN);
     OsSignal (SIGHUP, AutoResetServer);
 #endif
-    OsSignal (SIGINT, GiveUp);
+    OsSignal (SIGINT, AbortServer);
     OsSignal (SIGTERM, GiveUp);
     XFD_COPYSET (&WellKnownConnections, &AllSockets);
     ResetHosts(display);
@@ -824,11 +823,11 @@ EstablishNewConnections(ClientPtr clientUnused, pointer closure)
 {
     fd_set  readyconnections;     /* set of listeners that are ready */
     int curconn;                  /* fd of listener that's ready */
-    register int newconn;         /* fd of new client */
+    int newconn;         /* fd of new client */
     CARD32 connect_time;
-    register int i;
-    register ClientPtr client;
-    register OsCommPtr oc;
+    int i;
+    ClientPtr client;
+    OsCommPtr oc;
     fd_set tmask;
 
     XFD_ANDSET (&tmask, (fd_set*)closure, &WellKnownConnections);
