@@ -1,4 +1,3 @@
-/* $Xorg: auth.c,v 1.5 2001/02/09 02:05:40 xorgcvs Exp $ */
 /*
 
 Copyright 1988, 1998  The Open Group
@@ -26,7 +25,7 @@ other dealings in this Software without prior written authorization
 from The Open Group.
 
 */
-/* $XFree86: xc/programs/xdm/auth.c,v 3.36 2004/04/03 22:26:26 dawes Exp $ */
+/* $XFree86: xc/programs/xdm/auth.c,v 3.38 2006/02/20 00:14:38 dawes Exp $ */
 
 /*
  * xdm - display manager daemon
@@ -944,7 +943,6 @@ DefineSelf (int fd, FILE *file, Xauth *auth)
     int 		family;
     register ifr_type  *ifr;
 #ifdef USE_SIOCGLIFCONF
-    int			n;
     void *		bufptr = buf;
     size_t		buflen = sizeof(buf);
     struct lifconf	ifc;
