@@ -1,11 +1,9 @@
-/*
- *	$Xorg: input.c,v 1.3 2000/08/17 19:55:08 cpqbld Exp $
- */
+/* $XTermId: input.c,v 1.175 2006/02/12 22:45:50 tom Exp $ */
 
-/* $XFree86: xc/programs/xterm/input.c,v 3.67 2003/12/03 00:22:57 dawes Exp $ */
+/* $XFree86: xc/programs/xterm/input.c,v 3.75 2006/02/13 01:14:59 dickey Exp $ */
 
 /*
- * Copyright 1999-2001,2002 by Thomas E. Dickey
+ * Copyright 1999-2005,2006 by Thomas E. Dickey
  *
  *                         All Rights Reserved
  *
@@ -66,11 +64,11 @@
 #include <X11/keysymdef.h>
 #endif
 
-#ifdef HAVE_X11_DECKEYSYM_H
+#if HAVE_X11_DECKEYSYM_H
 #include <X11/DECkeysym.h>
 #endif
 
-#ifdef HAVE_X11_SUNKEYSYM_H
+#if HAVE_X11_SUNKEYSYM_H
 #include <X11/Sunkeysym.h>
 #endif
 
@@ -150,7 +148,7 @@ AdjustAfterInput(TScreen * screen)
 }
 
 /* returns true if the key is on the editing keypad */
-static Boolean
+static Bool
 IsEditFunctionKey(KeySym keysym)
 {
     switch (keysym) {
@@ -255,47 +253,6 @@ TranslateFromSUNPC(KeySym keysym)
 #define MODIFIER_PARM		/*nothing */
 #endif
 
-#if OPT_WIDE_CHARS
-/* Convert a Unicode value c into a UTF-8 sequence in strbuf */
-int
-convertFromUTF8(unsigned long c, Char * strbuf)
-{
-    int nbytes = 0;
-
-    if (c < 0x80) {
-	strbuf[nbytes++] = c;
-    } else if (c < 0x800) {
-	strbuf[nbytes++] = 0xc0 | (c >> 6);
-	strbuf[nbytes++] = 0x80 | (c & 0x3f);
-    } else if (c < 0x10000) {
-	strbuf[nbytes++] = 0xe0 | (c >> 12);
-	strbuf[nbytes++] = 0x80 | ((c >> 6) & 0x3f);
-	strbuf[nbytes++] = 0x80 | (c & 0x3f);
-    } else if (c < 0x200000) {
-	strbuf[nbytes++] = 0xf0 | (c >> 18);
-	strbuf[nbytes++] = 0x80 | ((c >> 12) & 0x3f);
-	strbuf[nbytes++] = 0x80 | ((c >> 6) & 0x3f);
-	strbuf[nbytes++] = 0x80 | (c & 0x3f);
-    } else if (c < 0x4000000) {
-	strbuf[nbytes++] = 0xf8 | (c >> 24);
-	strbuf[nbytes++] = 0x80 | ((c >> 18) & 0x3f);
-	strbuf[nbytes++] = 0x80 | ((c >> 12) & 0x3f);
-	strbuf[nbytes++] = 0x80 | ((c >> 6) & 0x3f);
-	strbuf[nbytes++] = 0x80 | (c & 0x3f);
-    } else if (c < UCS_LIMIT) {
-	strbuf[nbytes++] = 0xfe | (c >> 30);
-	strbuf[nbytes++] = 0x80 | ((c >> 24) & 0x3f);
-	strbuf[nbytes++] = 0x80 | ((c >> 18) & 0x3f);
-	strbuf[nbytes++] = 0x80 | ((c >> 12) & 0x3f);
-	strbuf[nbytes++] = 0x80 | ((c >> 6) & 0x3f);
-	strbuf[nbytes++] = 0x80 | (c & 0x3f);
-    } else
-	return convertFromUTF8(UCS_REPL, strbuf);
-
-    return nbytes;
-}
-#endif /* OPT_WIDE_CHARS */
-
 /*
  * Determine if we use the \E[3~ sequence for Delete, or the legacy ^?.  We
  * maintain the delete_is_del value as 3 states:  unspecified(2), true and
@@ -306,11 +263,11 @@ convertFromUTF8(unsigned long c, Char * strbuf)
  * setting, popup menu or escape sequence, it overrides the keyboard type
  * rather than the reverse.
  */
-Boolean
+Bool
 xtermDeleteIsDEL(void)
 {
     TScreen *screen = &term->screen;
-    Boolean result = True;
+    Bool result = True;
 
     if (term->keyboard.type == keyboardIsDefault
 	|| term->keyboard.type == keyboardIsVT220)
@@ -338,7 +295,7 @@ Input(TKeyboard * keyboard,
 
     char strbuf[STRBUFSIZE];
     Char *string;
-    int key = FALSE;
+    int key = False;
     int pty = screen->respond;
     int nbytes;
     KeySym keysym = 0;
@@ -354,8 +311,13 @@ Input(TKeyboard * keyboard,
 #if OPT_TCAP_QUERY
     if (screen->tc_query >= 0) {
 	keysym = screen->tc_query;
-	nbytes = 0;
-	strbuf[0] = 0;
+	if (keysym != XK_BackSpace) {
+	    nbytes = 0;
+	    strbuf[0] = 0;
+	} else {
+	    nbytes = 1;
+	    strbuf[0] = 8;
+	}
     } else
 #endif
 #if OPT_I18N_SUPPORT
@@ -468,7 +430,7 @@ Input(TKeyboard * keyboard,
 	&& keyboard->type != keyboardIsVT220
 #endif
 #if OPT_VT52_MODE
-	&& screen->ansi_level != 0
+	&& screen->vtXX_level != 0
 #endif
 	) {
 /*
@@ -566,12 +528,12 @@ Input(TKeyboard * keyboard,
 	VT52_CURSOR_KEYS;
 	MODIFIER_PARM;
 	unparseseq(&reply, pty);
-	key = TRUE;
+	key = True;
 #if 0				/* OPT_SUNPC_KBD should suppress - but only for vt220 compatibility */
     } else if (keyboard->type == keyboardIsVT220
-	       && screen->ansi_level <= 1
+	       && screen->vtXX_level <= 1
 	       && IsEditFunctionKey(keysym)) {
-	key = FALSE;		/* ignore editing-keypad in vt100 mode */
+	key = False;		/* ignore editing-keypad in vt100 mode */
 #endif
     } else if (IsCursorKey(keysym) &&
 	       keysym != XK_Prior && keysym != XK_Next) {
@@ -594,7 +556,7 @@ Input(TKeyboard * keyboard,
 	VT52_CURSOR_KEYS;
 	MODIFIER_PARM;
 	unparseseq(&reply, pty);
-	key = TRUE;
+	key = True;
     } else if (IsFunctionKey(keysym)
 	       || IsMiscFunctionKey(keysym)
 	       || IsEditFunctionKey(keysym)
@@ -657,7 +619,7 @@ Input(TKeyboard * keyboard,
 		&& (reply.a_nparam == 0 || reply.a_param[0] >= 0))
 		unparseseq(&reply, pty);
 	}
-	key = TRUE;
+	key = True;
     } else if (IsKeypadKey(keysym)) {
 	if (keypad_mode) {
 	    reply.a_type = SS3;
@@ -668,7 +630,7 @@ Input(TKeyboard * keyboard,
 	} else {
 	    unparseputc(kypd_num[keysym - XK_KP_Space], pty);
 	}
-	key = TRUE;
+	key = True;
     } else if (nbytes > 0) {
 #if OPT_TEK4014
 	if (screen->TekGIN) {
@@ -693,11 +655,27 @@ Input(TKeyboard * keyboard,
 	    }
 #endif
 	    if (eightbit && screen->input_eight_bits) {
-		if (CharOf(*string) < 128) {
-		    TRACE(("...input shift from %d to %d\n",
-			   CharOf(*string),
-			   CharOf(*string) | 0x80));
+		IChar ch = CharOf(*string);
+		if (ch < 128) {
 		    *string |= 0x80;
+		    TRACE(("...input shift from %d to %d (%#x to %#x)\n",
+			   ch, CharOf(*string),
+			   ch, CharOf(*string)));
+#if OPT_WIDE_CHARS
+		    if (screen->utf8_mode) {
+			/*
+			 * We could interpret the incoming code as "in the
+			 * current locale", but it's simpler to treat it as
+			 * a Unicode value to translate to UTF-8.
+			 */
+			ch = CharOf(*string);
+			nbytes = 2;
+			string[0] = 0xc0 | ((ch >> 6) & 0x3);
+			string[1] = 0x80 | (ch & 0x3f);
+			TRACE(("...encoded %#x in UTF-8 as %#x,%#x\n",
+			       ch, string[0], string[1]));
+		    }
+#endif
 		}
 		eightbit = False;
 	    }
@@ -721,7 +699,7 @@ Input(TKeyboard * keyboard,
 	}
 	while (nbytes-- > 0)
 	    unparseputc(*string++, pty);
-	key = TRUE;
+	key = True;
     }
     if (key && !TEK4014_ACTIVE(screen))
 	AdjustAfterInput(screen);
@@ -1007,7 +985,7 @@ TranslationsUseKeyword(Widget w, const char *keyword)
     Bool result = False;
 
     XtGetSubresources(w,
-		      (XtPointer) & data,
+		      (XtPointer) &data,
 		      "vt100",
 		      "VT100",
 		      key_resources,
@@ -1071,37 +1049,43 @@ VTInitModifiers(void)
     int min_keycode, max_keycode, keysyms_per_keycode = 0;
 
     if (keymap != 0) {
+	KeySym *theMap;
 
 	TRACE(("VTInitModifiers\n"));
 
 	XDisplayKeycodes(dpy, &min_keycode, &max_keycode);
-	XGetKeyboardMapping(dpy, min_keycode, (max_keycode - min_keycode + 1),
-			    &keysyms_per_keycode);
+	theMap = XGetKeyboardMapping(dpy,
+				     min_keycode,
+				     (max_keycode - min_keycode + 1),
+				     &keysyms_per_keycode);
 
-	for (i = k = 0, mask = 1; i < 8; i++, mask <<= 1) {
-	    for (j = 0; j < keymap->max_keypermod; j++) {
-		KeyCode code = keymap->modifiermap[k];
-		if (code != 0) {
-		    KeySym keysym;
-		    int l = 0;
-		    do {
-			keysym = XKeycodeToKeysym(dpy, code, l);
-			l++;
-		    } while (!keysym && l < keysyms_per_keycode);
-		    if (keysym == XK_Num_Lock) {
-			SaveMask(num_lock);
-		    } else if (keysym == XK_Alt_L) {
-			SaveMask(alt_left);
-		    } else if (keysym == XK_Alt_R) {
-			SaveMask(alt_right);
-		    } else if (keysym == XK_Meta_L) {
-			SaveMask(meta_left);
-		    } else if (keysym == XK_Meta_R) {
-			SaveMask(meta_right);
+	if (theMap != 0) {
+	    for (i = k = 0, mask = 1; i < 8; i++, mask <<= 1) {
+		for (j = 0; j < keymap->max_keypermod; j++) {
+		    KeyCode code = keymap->modifiermap[k];
+		    if (code != 0) {
+			KeySym keysym;
+			int l = 0;
+			do {
+			    keysym = XKeycodeToKeysym(dpy, code, l);
+			    l++;
+			} while (!keysym && l < keysyms_per_keycode);
+			if (keysym == XK_Num_Lock) {
+			    SaveMask(num_lock);
+			} else if (keysym == XK_Alt_L) {
+			    SaveMask(alt_left);
+			} else if (keysym == XK_Alt_R) {
+			    SaveMask(alt_right);
+			} else if (keysym == XK_Meta_L) {
+			    SaveMask(meta_left);
+			} else if (keysym == XK_Meta_R) {
+			    SaveMask(meta_right);
+			}
 		    }
+		    k++;
 		}
-		k++;
 	    }
+	    XFree(theMap);
 	}
 
 	/* Don't disable any mods if "alwaysUseMods" is true. */
@@ -1156,7 +1140,7 @@ hex2int(int c)
  * Returns the (shift, control) state in *state.
  */
 int
-xtermcapKeycode(char *params, unsigned *state)
+xtermcapKeycode(char **params, unsigned *state)
 {
     /* *INDENT-OFF* */
 #define DATA(tc,ti,x,y) { tc, ti, x, y }
@@ -1241,25 +1225,31 @@ xtermcapKeycode(char *params, unsigned *state)
     unsigned len = 0;
     int code = -1;
 #define MAX_TNAME_LEN 6
-    char name[MAX_TNAME_LEN];
+    char name[MAX_TNAME_LEN + 1];
     char *p;
 
+    TRACE(("xtermcapKeycode(%s)\n", *params));
+
     /* Convert hex encoded name to ascii */
-    for (p = params; hex2int(p[0]) >= 0 && hex2int(p[1]) >= 0; p += 2) {
-	if (len == MAX_TNAME_LEN - 1)
-	    return -1;
+    for (p = *params; hex2int(p[0]) >= 0 && hex2int(p[1]) >= 0; p += 2) {
+	if (len >= MAX_TNAME_LEN)
+	    break;
 	name[len++] = (hex2int(p[0]) << 4) + hex2int(p[1]);
     }
-    if (*p)
-	return -1;
     name[len] = 0;
-    for (n = 0; n < XtNumber(table); n++) {
-	if (!strcmp(table[n].ti, name) || !strcmp(table[n].tc, name)) {
-	    code = table[n].code;
-	    *state = table[n].state;
-	    break;
+    *params = p;
+
+    if (*p == 0 || *p == ';') {
+	for (n = 0; n < XtNumber(table); n++) {
+	    if (!strcmp(table[n].ti, name) || !strcmp(table[n].tc, name)) {
+		code = table[n].code;
+		*state = table[n].state;
+		break;
+	    }
 	}
     }
+
+    TRACE(("... xtermcapKeycode(%s, %u) -> %#06x\n", name, *state, code));
     return code;
 }
 #endif
