@@ -1,14 +1,10 @@
-/* $XTermId: misc.c,v 1.247 2005/01/29 22:05:32 tom Exp $ */
+/* $XTermId: misc.c,v 1.296 2006/04/02 21:35:06 tom Exp $ */
 
-/*
- *	$Xorg: misc.c,v 1.3 2000/08/17 19:55:09 cpqbld Exp $
- */
-
-/* $XFree86: xc/programs/xterm/misc.c,v 3.95 2005/01/29 22:17:32 dickey Exp $ */
+/* $XFree86: xc/programs/xterm/misc.c,v 3.106 2006/04/10 00:34:37 dickey Exp $ */
 
 /*
  *
- * Copyright 1999-2004,2005 by Thomas E. Dickey
+ * Copyright 1999-2005,2006 by Thomas E. Dickey
  *
  *                        All Rights Reserved
  *
@@ -75,10 +71,17 @@
 #include <X11/Xmu/Error.h>
 #include <X11/Xmu/SysUtil.h>
 #include <X11/Xmu/WinUtil.h>
+#include <X11/Xmu/Xmu.h>
 #if HAVE_X11_SUNKEYSYM_H
 #include <X11/Sunkeysym.h>
 #endif
 
+#ifdef HAVE_LANGINFO_CODESET
+#include <langinfo.h>
+#endif
+
+#include <xutf8.h>
+
 #include <data.h>
 #include <error.h>
 #include <menu.h>
@@ -145,6 +148,9 @@ xevents(void)
     XtInputMask input_mask;
     TScreen *screen = &term->screen;
 
+    if (need_cleanup)
+	Cleanup(0);
+
     if (screen->scroll_amt)
 	FlushScroll(screen);
     /*
@@ -197,7 +203,7 @@ xevents(void)
 		 )
 		 && event.xany.type == MotionNotify
 		 && event.xcrossing.window == XtWindow(term)) {
-	    SendMousePosition((Widget) term, &event);
+	    SendMousePosition(term, &event);
 	    continue;
 	}
 
@@ -321,15 +327,15 @@ HandleInterpret(Widget w GCC_UNUSED,
     if (*param_count == 1) {
 	char *value = params[0];
 	int need = strlen(value);
-	int used = VTbuffer.next - VTbuffer.buffer;
-	int have = VTbuffer.last - VTbuffer.buffer;
+	int used = VTbuffer->next - VTbuffer->buffer;
+	int have = VTbuffer->last - VTbuffer->buffer;
 
-	if (have - used + need < (int) sizeof(VTbuffer.buffer)) {
+	if (have - used + need < BUF_SIZE) {
 
-	    fillPtyData(&term->screen, &VTbuffer, value, (int) strlen(value));
+	    fillPtyData(&term->screen, VTbuffer, value, (int) strlen(value));
 
 	    TRACE(("Interpret %s\n", value));
-	    VTbuffer.update++;
+	    VTbuffer->update++;
 	}
     }
 }
@@ -339,6 +345,7 @@ DoSpecialEnterNotify(XEnterWindowEvent * ev)
 {
     TScreen *screen = &term->screen;
 
+    TRACE(("DoSpecialEnterNotify(%d)\n", screen->select));
 #ifdef ACTIVEWINDOWINPUTONLY
     if (ev->window == XtWindow(XtParent(CURRENT_EMU(screen))))
 #endif
@@ -356,6 +363,7 @@ HandleEnterWindow(Widget w GCC_UNUSED,
 		  Boolean * cont GCC_UNUSED)
 {
     /* NOP since we handled it above */
+    TRACE(("HandleEnterWindow ignored\n"));
 }
 
 static void
@@ -363,6 +371,7 @@ DoSpecialLeaveNotify(XEnterWindowEvent * ev)
 {
     TScreen *screen = &term->screen;
 
+    TRACE(("DoSpecialLeaveNotify(%d)\n", screen->select));
 #ifdef ACTIVEWINDOWINPUTONLY
     if (ev->window == XtWindow(XtParent(CURRENT_EMU(screen))))
 #endif
@@ -380,6 +389,7 @@ HandleLeaveWindow(Widget w GCC_UNUSED,
 		  Boolean * cont GCC_UNUSED)
 {
     /* NOP since we handled it above */
+    TRACE(("HandleLeaveWindow ignored\n"));
 }
 
 /*ARGSUSED*/
@@ -392,10 +402,28 @@ HandleFocusChange(Widget w GCC_UNUSED,
     XFocusChangeEvent *event = (XFocusChangeEvent *) ev;
     TScreen *screen = &term->screen;
 
+    TRACE(("HandleFocusChange type=%d, mode=%d, detail=%d\n",
+	   event->type,
+	   event->mode,
+	   event->detail));
+
     if (event->type == FocusIn) {
+	/*
+	 * NotifyNonlinear only happens (on FocusIn) if the pointer was not in
+	 * one of our windows.  Use this to reset a case where one xterm is
+	 * partly obscuring another, and X gets (us) confused about whether the
+	 * pointer was in the window.  In particular, this can happen if the
+	 * user is resizing the obscuring window, causing some events to not be
+	 * delivered to the obscured window.
+	 */
+	if (event->detail == NotifyNonlinear
+	    && (screen->select & INWINDOW) != 0) {
+	    unselectwindow(screen, INWINDOW);
+	}
 	selectwindow(screen,
-		     (event->detail == NotifyPointer) ? INWINDOW :
-		     FOCUS);
+		     ((event->detail == NotifyPointer)
+		      ? INWINDOW
+		      : FOCUS));
     } else {
 	/*
 	 * XGrabKeyboard() will generate FocusOut/NotifyGrab event that we want
@@ -403,8 +431,9 @@ HandleFocusChange(Widget w GCC_UNUSED,
 	 */
 	if (event->mode != NotifyGrab) {
 	    unselectwindow(screen,
-			   (event->detail == NotifyPointer) ? INWINDOW :
-			   FOCUS);
+			   ((event->detail == NotifyPointer)
+			    ? INWINDOW
+			    : FOCUS));
 	}
 	if (screen->grabbedKbd && (event->mode == NotifyUngrab)) {
 	    Bell(XkbBI_Info, 100);
@@ -418,6 +447,8 @@ HandleFocusChange(Widget w GCC_UNUSED,
 static void
 selectwindow(TScreen * screen, int flag)
 {
+    TRACE(("selectwindow(%d) flag=%d\n", screen->select, flag));
+
 #if OPT_TEK4014
     if (screen->TekEmu) {
 	if (!Ttoggled)
@@ -444,6 +475,8 @@ selectwindow(TScreen * screen, int flag)
 static void
 unselectwindow(TScreen * screen, int flag)
 {
+    TRACE(("unselectwindow(%d) flag=%d\n", screen->select, flag));
+
     if (screen->always_highlight)
 	return;
 
@@ -477,6 +510,9 @@ Bell(int which GCC_UNUSED, int percent)
     long now_msecs;
 
     TRACE(("BELL %d\n", percent));
+    if (!XtIsRealized((Widget) term)) {
+	return;
+    }
 
     /* has enough time gone by that we are allowed to ring
        the bell again? */
@@ -624,7 +660,7 @@ dabbrev_prev_char(int *xp, int *yp, TScreen * screen)
 	    return linep[*xp];
 	if (--*yp < 0)		/* go to previous line */
 	    break;
-	*xp = screen->max_col + 1;
+	*xp = MaxCols(screen);
 	if (!((long) BUF_FLAGS(screen->allbuf, *yp) & LINEWRAPPED))
 	    return ' ';		/* treat lines as separate */
     }
@@ -766,8 +802,9 @@ HandleIconify(Widget gw,
 }
 
 int
-QueryMaximize(TScreen * screen, unsigned *width, unsigned *height)
+QueryMaximize(XtermWidget termw, unsigned *width, unsigned *height)
 {
+    TScreen *screen = &termw->screen;
     XSizeHints hints;
     long supp = 0;
     Window root_win;
@@ -777,7 +814,7 @@ QueryMaximize(TScreen * screen, unsigned *width, unsigned *height)
     unsigned root_depth;
 
     if (XGetGeometry(screen->display,
-		     XDefaultRootWindow(screen->display),
+		     RootWindowOfScreen(XtScreen(termw)),
 		     &root_win,
 		     &root_x,
 		     &root_y,
@@ -825,7 +862,7 @@ RequestMaximize(XtermWidget termw, int maximize)
 
     if (maximize) {
 
-	if (QueryMaximize(screen, &root_width, &root_height)) {
+	if (QueryMaximize(termw, &root_width, &root_height)) {
 
 	    if (XGetWindowAttributes(screen->display,
 				     WMFrameWindow(termw),
@@ -954,7 +991,7 @@ timestamp_filename(char *dst, const char *src)
     time_t tstamp;
     struct tm *tstruct;
 
-    time(&tstamp);
+    tstamp = time((time_t *) 0);
     tstruct = localtime(&tstamp);
     sprintf(dst, TIMESTAMP_FMT,
 	    src,
@@ -1162,7 +1199,7 @@ StartLog(TScreen * screen)
 	    time_t now;
 	    struct tm *ltm;
 
-	    (void) time(&now);
+	    now = time((time_t *) 0);
 	    ltm = (struct tm *) localtime(&now);
 	    if ((gethostname(hostname, sizeof(hostname)) == 0) &&
 		(strftime(yyyy_mm_dd_hh_mm_ss,
@@ -1252,7 +1289,7 @@ StartLog(TScreen * screen)
 	    return;
     }
 #endif /*VMS */
-    screen->logstart = VTbuffer.next;
+    screen->logstart = VTbuffer->next;
     screen->logging = True;
     update_logging();
 }
@@ -1281,12 +1318,12 @@ FlushLog(TScreen * screen)
 	    return;
 	tt_new_output = False;
 #endif /* VMS */
-	cp = VTbuffer.next;
+	cp = VTbuffer->next;
 	if (screen->logstart != 0
 	    && (i = cp - screen->logstart) > 0) {
 	    write(screen->logfd, (char *) screen->logstart, (unsigned) i);
 	}
-	screen->logstart = VTbuffer.next;
+	screen->logstart = VTbuffer->next;
     }
 }
 
@@ -1506,6 +1543,85 @@ ChangeAnsiColorRequest(XtermWidget pTerm,
 #define find_closest_color(display, cmap, def) 0
 #endif /* OPT_ISO_COLORS */
 
+#if OPT_PASTE64
+static void
+ManipulateSelectionData(TScreen * screen, char *buf, int final)
+{
+#define PDATA(a,b) { a, #b }
+    static struct {
+	char given;
+	char *result;
+    } table[] = {
+	PDATA('s', SELECT),
+	    PDATA('p', PRIMARY),
+	    PDATA('c', CLIPBOARD),
+	    PDATA('0', CUT_BUFFER0),
+	    PDATA('1', CUT_BUFFER1),
+	    PDATA('2', CUT_BUFFER2),
+	    PDATA('3', CUT_BUFFER3),
+	    PDATA('4', CUT_BUFFER4),
+	    PDATA('5', CUT_BUFFER5),
+	    PDATA('6', CUT_BUFFER6),
+	    PDATA('7', CUT_BUFFER7),
+    };
+
+    char *base = buf;
+    char *used = x_strdup(base);
+    Cardinal j, n = 0;
+    char **select_args = 0;
+
+    TRACE(("Manipulate selection data\n"));
+
+    while (*buf != ';' && *buf != '\0') {
+	++buf;
+    }
+
+    if (*buf == ';') {
+	*buf++ = '\0';
+
+	if (*base == '\0')
+	    base = "s0";
+	if ((select_args = TypeCallocN(String, 1 + strlen(base))) == 0)
+	    return;
+	while (*base != '\0') {
+	    for (j = 0; j < XtNumber(table); ++j) {
+		if (*base == table[j].given) {
+		    used[n] = *base;
+		    select_args[n++] = table[j].result;
+		    TRACE(("atom[%d] %s\n", n, table[j].result));
+		    break;
+		}
+	    }
+	    ++base;
+	}
+	used[n] = 0;
+
+	if (!strcmp(buf, "?")) {
+	    TRACE(("Getting selection\n"));
+	    unparseputc1(OSC, screen->respond);
+	    unparseputs("52", screen->respond);
+	    unparseputc(';', screen->respond);
+
+	    unparseputs(used, screen->respond);
+	    unparseputc(';', screen->respond);
+
+	    /* Tell xtermGetSelection data is base64 encoded */
+	    screen->base64_paste = n;
+	    screen->base64_final = final;
+
+	    /* terminator will be written in this call */
+	    xtermGetSelection((Widget) term, 0, select_args, n, NULL);
+	} else {
+	    TRACE(("Setting selection with %s\n", buf));
+	    ClearSelectionBuffer(screen);
+	    while (*buf != '\0')
+		AppendToSelectionBuffer(screen, CharOf(*buf++));
+	    CompleteSelection(term, select_args, n);
+	}
+    }
+}
+#endif /* OPT_PASTE64 */
+
 /***====================================================================***/
 
 void
@@ -1517,6 +1633,8 @@ do_osc(Char * oscbuf, unsigned len GCC_UNUSED, int final)
     int state = 0;
     char *buf = 0;
 
+    TRACE(("do_osc %s\n", oscbuf));
+
     /*
      * Lines should be of the form <OSC> number ; string <ST>, however
      * older xterms can accept <BEL> as a final character.  We will respond
@@ -1530,14 +1648,18 @@ do_osc(Char * oscbuf, unsigned len GCC_UNUSED, int final)
 	case 0:
 	    if (isdigit(*cp)) {
 		mode = 10 * mode + (*cp - '0');
-		if (mode > 65535)
+		if (mode > 65535) {
+		    TRACE(("do_osc found unknown mode %d\n", mode));
 		    return;
+		}
 		break;
 	    }
 	    /* FALLTHRU */
 	case 1:
-	    if (*cp != ';')
+	    if (*cp != ';') {
+		TRACE(("do_osc did not find semicolon offset %d\n", cp - oscbuf));
 		return;
+	    }
 	    state = 2;
 	    break;
 	case 2:
@@ -1545,8 +1667,26 @@ do_osc(Char * oscbuf, unsigned len GCC_UNUSED, int final)
 	    state = 3;
 	    /* FALLTHRU */
 	default:
-	    if (ansi_table[CharOf(*cp)] != CASE_PRINT)
-		return;
+	    if (ansi_table[CharOf(*cp)] != CASE_PRINT) {
+		switch (mode) {
+		case 0:
+		case 1:
+		case 2:
+#if OPT_WIDE_CHARS
+		    /*
+		     * If we're running with UTF-8, it is possible for title
+		     * strings to contain "nonprinting" text.
+		     */
+		    if (xtermEnvUTF8())
+#endif
+			break;
+		default:
+		    TRACE(("do_osc found nonprinting char %02X offset %d\n",
+			   CharOf(*cp),
+			   cp - oscbuf));
+		    return;
+		}
+	    }
 	}
     }
     if (buf == 0)
@@ -1625,7 +1765,7 @@ do_osc(Char * oscbuf, unsigned len GCC_UNUSED, int final)
 	    unparseputc1(OSC, screen->respond);
 	    unparseputs("50", screen->respond);
 
-	    if ((buf = screen->menu_font_names[num]) != 0) {
+	    if ((buf = screen->MenuFontName(num)) != 0) {
 		unparseputc(';', screen->respond);
 		unparseputs(buf, screen->respond);
 	    }
@@ -1670,19 +1810,25 @@ do_osc(Char * oscbuf, unsigned len GCC_UNUSED, int final)
 
 		if (num < 0
 		    || num > fontMenu_lastBuiltin
-		    || (buf = screen->menu_font_names[num]) == 0) {
+		    || (buf = screen->MenuFontName(num)) == 0) {
 		    Bell(XkbBI_MinorError, 0);
 		    break;
 		}
 	    }
 	    fonts.f_n = buf;
-	    SetVTFont(fontMenu_fontescape, True, &fonts);
+	    SetVTFont(term, fontMenu_fontescape, True, &fonts);
 	}
 	break;
     case 51:
-	/* reserved for Emacs shell (Rob Myoff <mayoff@dqd.com>) */
+	/* reserved for Emacs shell (Rob Mayoff <mayoff@dqd.com>) */
 	break;
 
+#if OPT_PASTE64
+    case 52:
+	if (screen->allowWindowOps && (buf != 0))
+	    ManipulateSelectionData(screen, buf, final);
+	break;
+#endif
 	/*
 	 * One could write code to send back the display and host names,
 	 * but that could potentially open a fairly nasty security hole.
@@ -2038,38 +2184,56 @@ do_dcs(Char * dcsbuf, size_t dcslen)
 	    unsigned state;
 	    int code;
 	    char *tmp;
+	    char *parsed = ++cp;
 
-	    ++cp;
-	    code = xtermcapKeycode(cp, &state);
 	    unparseputc1(DCS, screen->respond);
+
+	    code = xtermcapKeycode(&parsed, &state);
 	    unparseputc(code >= 0 ? '1' : '0', screen->respond);
+
 	    unparseputc('+', screen->respond);
 	    unparseputc('r', screen->respond);
-	    for (tmp = cp; *tmp; ++tmp)
-		unparseputc(*tmp, screen->respond);
-	    if (code >= 0) {
-		unparseputc('=', screen->respond);
-		screen->tc_query = code;
-		/* XK_COLORS is a fake code for the "Co" entry (maximum
-		 * number of colors) */
-		if (code == XK_COLORS) {
+
+	    while (*cp != 0) {
+		if (cp == parsed)
+		    break;	/* no data found, error */
+
+		for (tmp = cp; tmp != parsed; ++tmp)
+		    unparseputc(*tmp, screen->respond);
+
+		if (code >= 0) {
+		    unparseputc('=', screen->respond);
+		    screen->tc_query = code;
+		    /* XK_COLORS is a fake code for the "Co" entry (maximum
+		     * number of colors) */
+		    if (code == XK_COLORS) {
 # if OPT_256_COLORS
-		    unparseputc('2', screen->respond);
-		    unparseputc('5', screen->respond);
-		    unparseputc('6', screen->respond);
+			unparseputc('2', screen->respond);
+			unparseputc('5', screen->respond);
+			unparseputc('6', screen->respond);
 # elif OPT_88_COLORS
-		    unparseputc('8', screen->respond);
-		    unparseputc('8', screen->respond);
+			unparseputc('8', screen->respond);
+			unparseputc('8', screen->respond);
 # else
-		    unparseputc('1', screen->respond);
-		    unparseputc('6', screen->respond);
+			unparseputc('1', screen->respond);
+			unparseputc('6', screen->respond);
 # endif
+		    } else {
+			XKeyEvent event;
+			event.state = state;
+			Input(&(term->keyboard), screen, &event, False);
+		    }
+		    screen->tc_query = -1;
 		} else {
-		    XKeyEvent event;
-		    event.state = state;
-		    Input(&(term->keyboard), screen, &event, False);
+		    break;	/* no match found, error */
+		}
+
+		cp = parsed;
+		if (*parsed == ';') {
+		    unparseputc(*parsed++, screen->respond);
+		    cp = parsed;
+		    code = xtermcapKeycode(&parsed, &state);
 		}
-		screen->tc_query = -1;
 	    }
 	    unparseputc1(ST, screen->respond);
 	}
@@ -2104,24 +2268,94 @@ udk_lookup(int keycode, int *len)
 static void
 ChangeGroup(String attribute, char *value)
 {
+#if OPT_WIDE_CHARS
+    static Char *converted;	/* NO_LEAKS */
+#endif
     Arg args[1];
-    const char *name = (value != 0) ? (char *) value : "";
+    char *original = (value != 0) ? value : "";
+    char *name = original;
+    TScreen *screen = &term->screen;
+    Widget w = CURRENT_EMU(screen);
+    Widget top = SHELL_OF(w);
+    unsigned limit = strlen(name);
 
     TRACE(("ChangeGroup(attribute=%s, value=%s)\n", attribute, name));
+
+    /*
+     * Ignore titles that are too long to be plausible requests.
+     */
+    if (limit >= 1024)
+	return;
+
+#if OPT_WIDE_CHARS
+    /*
+     * Title strings are limited to ISO-8859-1, which is consistent with the
+     * printable data in sos_table.  However, if we're running in UTF-8 mode,
+     * it is likely that non-ASCII text in the string will be rejected because
+     * it is not printable in the current locale.  So we convert it to UTF-8,
+     * allowing the X library to convert it back.
+     */
+    if (xtermEnvUTF8() && !screen->utf8_title) {
+	int n;
+
+	for (n = 0; name[n] != '\0'; ++n) {
+	    if (CharOf(name[n]) > 127) {
+		if (converted != 0)
+		    free(converted);
+		if ((converted = TypeMallocN(Char, 1 + (5 * limit))) != 0) {
+		    Char *temp = converted;
+		    while (*name != 0) {
+			temp = convertToUTF8(temp, CharOf(*name));
+			++name;
+		    }
+		    *temp = 0;
+		    name = (char *) converted;
+		    TRACE(("...converted{%s}\n", name));
+		}
+		break;
+	    }
+	}
+    }
+#endif
+
 #if OPT_SAME_NAME
     /* If the attribute isn't going to change, then don't bother... */
 
     if (sameName) {
 	char *buf;
 	XtSetArg(args[0], attribute, &buf);
-	XtGetValues(toplevel, args, 1);
+	XtGetValues(top, args, 1);
+	TRACE(("...comparing{%s}\n", buf));
 	if (strcmp(name, buf) == 0)
 	    return;
     }
 #endif /* OPT_SAME_NAME */
 
+    TRACE(("...updating %s\n", attribute));
     XtSetArg(args[0], attribute, name);
-    XtSetValues(toplevel, args, 1);
+    XtSetValues(top, args, 1);
+
+#if OPT_WIDE_CHARS
+    if (xtermEnvUTF8()) {
+	Display *dpy = XtDisplay(term);
+	Atom my_atom;
+	char *propname = (!strcmp(attribute, XtNtitle)
+			  ? "_NET_WM_NAME"
+			  : "_NET_WM_ICON_NAME");
+	if ((my_atom = XInternAtom(dpy, propname, False)) != None) {
+	    if (screen->utf8_title) {
+		TRACE(("...updating %s\n", propname));
+		XChangeProperty(dpy, VShellWindow,
+				my_atom, XA_UTF8_STRING(dpy), 8,
+				PropModeReplace,
+				(Char *) original, strlen(original));
+	    } else {
+		TRACE(("...deleting %s\n", propname));
+		XDeleteProperty(dpy, VShellWindow, my_atom);
+	    }
+	}
+    }
+#endif
 }
 
 void
@@ -2285,6 +2519,10 @@ UpdateOldColors(XtermWidget pTerm GCC_UNUSED, ScrnColors * pNew)
     return (True);
 }
 
+/*
+ * This is part of ReverseVideo().  It reverses the data stored for the old
+ * "dynamic" colors that might have been retrieved using OSC 10-18.
+ */
 void
 ReverseOldColors(void)
 {
@@ -2301,9 +2539,7 @@ ReverseOldColors(void)
 		pOld->names[TEXT_CURSOR] = NULL;
 	    }
 	    if (pOld->names[TEXT_BG]) {
-		tmpName = XtMalloc(strlen(pOld->names[TEXT_BG]) + 1);
-		if (tmpName) {
-		    strcpy(tmpName, pOld->names[TEXT_BG]);
+		if ((tmpName = x_strdup(pOld->names[TEXT_BG])) != 0) {
 		    pOld->names[TEXT_CURSOR] = tmpName;
 		}
 	    }
@@ -2337,9 +2573,10 @@ AllocateTermColor(XtermWidget pTerm,
     if (XParseColor(screen->display, cmap, name, &def)
 	&& (XAllocColor(screen->display, cmap, &def)
 	    || find_closest_color(screen->display, cmap, &def))
-	&& (newName = XtMalloc(strlen(name) + 1)) != 0) {
+	&& (newName = x_strdup(name)) != 0) {
+	if (COLOR_DEFINED(pNew, ndx))
+	    free(pNew->names[ndx]);
 	SET_COLOR_VALUE(pNew, ndx, def.pixel);
-	strcpy(newName, name);
 	SET_COLOR_NAME(pNew, ndx, newName);
 	TRACE(("AllocateTermColor #%d: %s (pixel %#lx)\n", ndx, newName, def.pixel));
 	return (True);
@@ -2536,6 +2773,7 @@ Cleanup(int code)
 	}
 
 	cleaning = True;
+	need_cleanup = FALSE;
 
 	TRACE(("Cleanup %d\n", code));
 
@@ -2561,6 +2799,61 @@ Cleanup(int code)
     Exit(code);
 }
 
+#ifndef VMS
+char *
+xtermFindShell(char *leaf, Bool warning)
+{
+    char *s;
+    char *d;
+    char *tmp;
+    char *result = leaf;
+
+    TRACE(("xtermFindShell(%s)\n", leaf));
+    if (*result != '\0' && strchr("+/-", *result) == 0) {
+	/* find it in $PATH */
+	if ((s = getenv("PATH")) != 0) {
+	    if ((tmp = TypeMallocN(char, strlen(leaf) + strlen(s) + 1)) != 0) {
+		Bool found = False;
+		while (*s != '\0') {
+		    strcpy(tmp, s);
+		    for (d = tmp;; ++d) {
+			if (*d == ':' || *d == '\0') {
+			    int skip = (*d != '\0');
+			    *d = '/';
+			    strcpy(d + 1, leaf);
+			    if (skip)
+				++d;
+			    s += (d - tmp);
+			    if (*tmp == '/'
+				&& strstr(tmp, "..") == 0
+				&& access(tmp, X_OK) == 0) {
+				result = x_strdup(tmp);
+				found = True;
+			    }
+			    break;
+			}
+			if (found)
+			    break;
+		    }
+		    if (found)
+			break;
+		}
+		free(tmp);
+	    }
+	}
+    }
+    TRACE(("...xtermFindShell(%s)\n", result));
+    if (*result != '/'
+	|| strstr(result, "..") != 0
+	|| access(result, X_OK) != 0) {
+	if (warning)
+	    fprintf(stderr, "No absolute path found for shell: %s\n", result);
+	result = 0;
+    }
+    return result;
+}
+#endif /* VMS */
+
 /*
  * sets the value of var to be arg in the Unix 4.2 BSD environment env.
  * Var should end with '=' (bindings are of the form "var=value").
@@ -2571,28 +2864,30 @@ Cleanup(int code)
 void
 xtermSetenv(char *var, char *value)
 {
-    int envindex = 0;
-    size_t len = strlen(var);
+    if (value != 0) {
+	int envindex = 0;
+	size_t len = strlen(var);
 
-    TRACE(("xtermSetenv(var=%s, value=%s)\n", var, value));
+	TRACE(("xtermSetenv(var=%s, value=%s)\n", var, value));
 
-    while (environ[envindex] != NULL) {
-	if (strncmp(environ[envindex], var, len) == 0) {
-	    /* found it */
-	    environ[envindex] = CastMallocN(char, len + strlen(value));
-	    strcpy(environ[envindex], var);
-	    strcat(environ[envindex], value);
-	    return;
+	while (environ[envindex] != NULL) {
+	    if (strncmp(environ[envindex], var, len) == 0) {
+		/* found it */
+		environ[envindex] = CastMallocN(char, len + strlen(value));
+		strcpy(environ[envindex], var);
+		strcat(environ[envindex], value);
+		return;
+	    }
+	    envindex++;
 	}
-	envindex++;
+
+	TRACE(("...expanding env to %d\n", envindex + 1));
+
+	environ[envindex] = CastMallocN(char, len + strlen(value));
+	(void) strcpy(environ[envindex], var);
+	strcat(environ[envindex], value);
+	environ[++envindex] = NULL;
     }
-
-    TRACE(("...expanding env to %d\n", envindex + 1));
-
-    environ[envindex] = CastMallocN(char, len + strlen(value));
-    (void) strcpy(environ[envindex], var);
-    strcat(environ[envindex], value);
-    environ[++envindex] = NULL;
 }
 
 /*ARGSUSED*/
@@ -2672,6 +2967,7 @@ set_vt_visibility(Bool on)
 #if OPT_TOOLBAR
 	    /* we need both of these during initialization */
 	    XtMapWidget(SHELL_OF(term));
+	    ShowToolbar(resource.toolBar);
 #endif
 	    screen->Vshow = True;
 	}
@@ -2680,7 +2976,7 @@ set_vt_visibility(Bool on)
     else {
 	if (screen->Vshow && term) {
 	    withdraw_window(XtDisplay(term),
-			    XtWindow(SHELL_OF(term)),
+			    VShellWindow,
 			    XScreenNumberOfScreen(XtScreen(term)));
 	    screen->Vshow = False;
 	}
@@ -2722,7 +3018,7 @@ set_tek_visibility(Bool on)
     } else {
 	if (screen->Tshow && tekWidget) {
 	    withdraw_window(XtDisplay(tekWidget),
-			    XtWindow(SHELL_OF(tekWidget)),
+			    TShellWindow,
 			    XScreenNumberOfScreen(XtScreen(tekWidget)));
 	    screen->Tshow = False;
 	}
@@ -2823,6 +3119,12 @@ sortedOptDescs(XrmOptionDescRec * descs, Cardinal res_count)
 {
     static XrmOptionDescRec *res_array = 0;
 
+#ifdef NO_LEAKS
+    if (descs == 0 && res_array != 0) {
+	free(res_array);
+	res_array = 0;
+    } else
+#endif
     if (res_array == 0) {
 	Cardinal j;
 
@@ -2846,6 +3148,13 @@ sortedOpts(OptionHelp * options, XrmOptionDescRec * descs, Cardinal numDescs)
 {
     static OptionHelp *opt_array = 0;
 
+#ifdef NO_LEAKS
+    if (descs == 0 && opt_array != 0) {
+	sortedOptDescs(descs, numDescs);
+	free(opt_array);
+	opt_array = 0;
+    } else
+#endif
     if (opt_array == 0) {
 	Cardinal opt_count, j;
 #if OPT_TRACE
@@ -2923,6 +3232,70 @@ sortedOpts(OptionHelp * options, XrmOptionDescRec * descs, Cardinal numDescs)
     return opt_array;
 }
 
+/*
+ * Report the locale that xterm was started in.
+ */
+char *
+xtermEnvLocale(void)
+{
+    static char *result;
+
+    if (result == 0) {
+	if ((result = getenv("LC_ALL")) == 0 || *result == '\0')
+	    if ((result = getenv("LC_CTYPE")) == 0 || *result == '\0')
+		if ((result = getenv("LANG")) == 0 || *result == '\0')
+		    result = "";
+	TRACE(("xtermEnvLocale ->%s\n", result));
+    }
+    return result;
+}
+
+char *
+xtermEnvEncoding(void)
+{
+    static char *result;
+
+    if (result == 0) {
+#ifdef HAVE_LANGINFO_CODESET
+	result = nl_langinfo(CODESET);
+#else
+	char *locale = xtermEnvLocale();
+	if (*locale == 0 || !strcmp(locale, "C") || !strcmp(locale, "POSIX")) {
+	    result = "ASCII";
+	} else {
+	    result = "ISO-8859-1";
+	}
+#endif
+	TRACE(("xtermEnvEncoding ->%s\n", result));
+    }
+    return result;
+}
+
+#if OPT_WIDE_CHARS
+/*
+ * Tell whether xterm was started in a locale that uses UTF-8 encoding for
+ * characters.  That environment is inherited by subprocesses and used in
+ * various library calls.
+ */
+Bool
+xtermEnvUTF8(void)
+{
+    static Bool init = False;
+    static Bool result = False;
+
+    if (!init) {
+	init = True;
+#ifdef HAVE_LANGINFO_CODESET
+	result = (strcmp(xtermEnvEncoding(), "UTF-8") == 0);
+#else
+	result = (strstr(xtermEnvLocale(), "UTF-8") != NULL);
+#endif
+	TRACE(("xtermEnvUTF8 ->%s\n", BtoS(result)));
+    }
+    return result;
+}
+#endif /* OPT_WIDE_CHARS */
+
 /*
  * Returns the version-string used in the "-v' message as well as a few other
  * places.  It is derived (when possible) from the __vendorversion__ symbol
